# Generated by Django 4.2.10 on 2024-02-29 07:58

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0045_additionalmapboundarydata_active_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='colorvalue',
            name='show_on_legend',
            field=models.BooleanField(default=True, verbose_name='Show label on Legend'),
        ),
        migrations.AddField(
            model_name='rasterstyle',
            name='legend_type',
            field=models.CharField(choices=[('basic', 'Basic'), ('choropleth', 'Choropleth'), ('gradient', 'Gradient')], default='choropleth', max_length=100, verbose_name='Legend Type'),
        ),
    ]



from django.utils.translation import gettext_lazy as _
from wagtail.admin.forms import WagtailAdminModelForm


class RasterStyleModelForm(WagtailAdminModelForm):
    def is_valid(self):
        valid = super().is_valid()
        if not valid:
            return False

        use_custom_colors = self.cleaned_data.get("use_custom_colors")
        min_value = self.cleaned_data.get("min")
        max_value = self.cleaned_data.get("max")
        steps = self.cleaned_data.get("steps")
        custom_color_for_rest = self.cleaned_data.get("custom_color_for_rest")

        if max_value <= min_value:
            self.add_error("max", _("Maximum value should be greater than minimum value"))
            return False

        if min_value >= max_value:
            self.add_error("min", _("Minimum value should be less than maximum value"))
            return False

        if not use_custom_colors and not steps:
            self.add_error("steps", _("Steps required when not using custom colors"))
            return False

        if use_custom_colors:

            # no color for values greater than max
            if not custom_color_for_rest:
                self.add_error("custom_color_for_rest",
                               _("Color for the rest of values must be specified"))
                return False

            color_values_formset = self.formsets.get("color_values")

            initial_form_count = color_values_formset.initial_form_count()
            total_form_count = color_values_formset.total_form_count()
            deleted_forms_count = len(color_values_formset.deleted_forms)

            empty_forms_count = 0
            for i, form in enumerate(color_values_formset.forms):
                # Empty forms are unchanged forms beyond those with initial data.
                if not form.has_changed() and i >= initial_form_count:
                    empty_forms_count += 1

            # No custom value added
            if total_form_count - deleted_forms_count - empty_forms_count < 1:
                color_values_formset._non_form_errors = [
                    _("You selected to use custom colors but did not add any. Please add color values and save.")
                ]
                return False

            for form in color_values_formset:
                threshold = form.cleaned_data.get("threshold")

                if threshold and min_value and max_value:
                    if threshold < min_value:
                        form.add_error("threshold", _("Value must be greater than minimum defined value"))
                        return False
                    if threshold > max_value:
                        form.add_error("threshold", _("Value must be less than or equal to the maximum defined value"))
                        return False
        return True



# Generated by Django 4.1.10 on 2023-11-19 18:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0034_rastertilelayer_time_parameter_name_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='vectortilelayer',
            name='render_layers_json',
            field=models.JSONField(blank=True, null=True, verbose_name='Layers Configuration'),
        ),
        migrations.AddField(
            model_name='vectortilelayer',
            name='use_render_layers_json',
            field=models.BooleanField(default=False, verbose_name='Use Render Layers JSON'),
        ),
    ]



from django.utils.safestring import mark_safe
from django.utils.translation import gettext_lazy as _
from wagtail_adminsortable.admin import SortableAdminMixin
from wagtail_adminsortable.views import SortableIndexView
from wagtail_modeladmin.helpers import AdminURLHelper, ButtonHelper
from wagtail_modeladmin.views import CreateView, EditView

from geomanager.admin.base import BaseModelAdmin, ModelAdminCanHide
from geomanager.models import Category, Dataset, SubCategory


class DatasetIndexView(SortableIndexView):
    def get_context_data(self, **kwargs):
        context_data = super(DatasetIndexView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": "#", "label": Dataset._meta.verbose_name_plural},
        ]

        context_data.update({
            "custom_create_url": {
                "label": _("Create from categories"),
                "url": category_index_url
            },
            "navigation_items": navigation_items,
        })

        return context_data


class DatasetCreateView(CreateView):
    def get_form(self):
        form = super().get_form()
        category_id = self.request.GET.get("category_id")
        if category_id:
            # form.fields["category"].widget.attrs.update({"disabled": "true"})
            form.fields["sub_category"].queryset = SubCategory.objects.filter(category=category_id)
            initial = {**form.initial}
            initial.update({"category": category_id})
            form.initial = initial
        return form

    def get_context_data(self, **kwargs):
        context_data = super(DatasetCreateView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": "#", "label": _("New") + f" {Dataset._meta.verbose_name}"},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class DatasetEditView(EditView):
    def get_context_data(self, **kwargs):
        context_data = super(DatasetEditView, self).get_context_data(**kwargs)

        category_admin_helper = AdminURLHelper(Category)
        category_index_url = category_admin_helper.get_action_url("index")

        datasets_admin_helper = AdminURLHelper(Dataset)
        datasets_index_url = datasets_admin_helper.get_action_url("index")

        navigation_items = [
            {"url": category_index_url, "label": Category._meta.verbose_name_plural},
            {"url": datasets_index_url, "label": Dataset._meta.verbose_name_plural},
            {"url": "#", "label": self.instance.title},
        ]

        context_data.update({
            "navigation_items": navigation_items,
        })

        return context_data


class DatasetButtonHelper(ButtonHelper):
    def get_buttons_for_obj(
            self, obj, exclude=None, classnames_add=None, classnames_exclude=None
    ):
        buttons = super().get_buttons_for_obj(obj, exclude, classnames_add, classnames_exclude)

        classnames = self.edit_button_classnames + classnames_add
        cn = self.finalise_classname(classnames, classnames_exclude)

        layer_create_url = obj.create_layer_url()

        if layer_create_url:
            create_layer_button = {
                "url": obj.create_layer_url(),
                "label": _("Add Layer"),
                "classname": cn,
                "title": _("Add %(object)s Layer") % {"object": self.verbose_name},
            }
            buttons.append(create_layer_button)

        return buttons


class DatasetModelAdmin(SortableAdminMixin, BaseModelAdmin, ModelAdminCanHide):
    hidden = True
    model = Dataset
    exclude_from_explorer = True
    button_helper_class = DatasetButtonHelper
    list_display = ("__str__", "layer_type",)
    list_filter = ("category",)
    # index_template_name = "geomanager/modeladmin/index_without_custom_create.html"
    # index_template_name = 'adminsortable/index.html'
    menu_icon = "database"

    index_view_class = DatasetIndexView
    create_view_class = DatasetCreateView
    edit_view_class = DatasetEditView

    form_view_extra_js = ["geomanager/js/dataset-form-conditional.js"]

    def __init__(self, parent=None):
        super().__init__(parent)
        self.list_display = (list(self.list_display) or []) + ['view_layers', 'mapviewer_map_url']

        self.view_layers.__func__.short_description = _('Layers')
        self.mapviewer_map_url.__func__.short_description = _("View on MapViewer")

    def mapviewer_map_url(self, obj):
        label = _("MapViewer")

        if not obj.has_layers():
            return None

        button_html = f"""
                    <a href="{obj.mapviewer_map_url}" target="_blank" rel="noopener noreferrer" class="button button-small button--icon button-secondary">
                        <span class="icon-wrapper">
                            <svg class="icon icon-map icon" aria-hidden="true">
                                <use href="#icon-map"></use>
                            </svg>
                        </span>
                        {label}
                    </a>
                """
        return mark_safe(button_html)

    def category_link(self, obj):
        label = _("Edit Category")
        button_html = f"""
            <a href="{obj.category_url}">
                {label}
            </a>
        """
        return mark_safe(button_html)

    def add_layer(self, obj):
        label = _("Add Layer")
        button_html = f"""
            <a href="{obj.create_layer_url()}" class="button button-small bicolor button--icon ">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-plus"></use>
                    </svg>
                </span>
                {label}
            </a>
        """

        return mark_safe(button_html)

    def view_layers(self, obj):
        label = _("View Layers")

        if not obj.has_layers():
            return self.add_layer(obj)

        button_html = f"""
            <a href="{obj.layers_list_url()}" class="button button-small button--icon bicolor button-secondary">
                <span class="icon-wrapper">
                    <svg class="icon icon-layer-group icon" aria-hidden="true">
                        <use href="#icon-layer-group"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)

    def preview_dataset(self, obj):
        if not obj.preview_url:
            return None

        if obj.layer_type == "vector_file":
            return None

        disabled = "" if obj.can_preview() else "disabled"
        label = _("Preview Dataset")
        button_html = f"""
            <a href="{obj.preview_url}" class="button button-small button--icon button-secondary {disabled}">
                <span class="icon-wrapper">
                    <svg class="icon icon-plus icon" aria-hidden="true">
                        <use href="#icon-view"></use>
                    </svg>
                </span>
                {label}
            </a>
        """
        return mark_safe(button_html)



class Error(Exception):

    def __init__(self, message):
        self.message = message

    @property
    def serialize(self):
        return {
            'message': self.message
        }


class InvalidFile(Error):
    pass


class RasterConvertError(Error):
    pass


class UnsupportedRasterFormat(Error):
    pass


class NoShpFound(Error):
    pass


class NoShxFound(Error):
    pass


class NoDbfFound(Error):
    pass


# Tile GL

class MissingTileError(Error):
    pass


class MBTilesNotFoundError(Error):
    pass


class MBTilesInvalid(Error):
    pass


class MissingBoundaryField(Error):
    pass


class NoMatchingBoundaryLayer(Error):
    pass


class NoMatchingBoundaryData(Error):
    pass


class InvalidBoundaryGeomType(Error):
    pass


class RasterFileNotFound(Error):
    pass


class GeostoreNotFound(Error):
    pass


class QueryParamRequired(Error):
    pass


class InvalidGeomType(Error):
    pass


class GeomValidationNotImplemented(Error):
    pass



from django.forms import ModelForm

from geomanager.models.profile import GeoManagerUserProfile


class GeoManagerUserProfileForm(ModelForm):
    class Meta:
        model = GeoManagerUserProfile
        fields = "__all__"





# Generated by Django 4.2.1 on 2023-06-09 10:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0003_alter_colorvalue_label_alter_colorvalue_threshold_and_more'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataset',
            name='can_clip',
            field=models.BooleanField(default=True, help_text='Check to enable clipping by boundary or drawn shapes, for raster and vector datasets. Not implemented for WMS types', verbose_name='Enable Clipping by shape'),
        ),
    ]



# Generated by Django 4.2.18 on 2025-05-08 08:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0051_rasterstyle_rendering_engine'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rasterstyle',
            name='rendering_engine',
        ),
    ]



from django import forms
from django.utils.translation import gettext_lazy as _


class VectorLayerFileForm(forms.Form):
    layer = forms.ModelChoiceField(required=True, queryset=None, empty_label=None, label=_("layer"))
    time = forms.DateTimeField(required=True,
                               widget=forms.DateTimeInput(attrs={'type': 'datetime-local'}, ),
                               localize=False,
                               label=_("time"),
                               help_text=_("Time is required for every uploaded vector file. "
                                           "This can be the time the dataset was collected, or applies to."))
    table_name = forms.CharField(required=True, label=_("database table name"))
    description = forms.CharField(required=False, widget=forms.Textarea, label=_("dataset description"),
                                  help_text=_("optional dataset description"))

    def __init__(self, *args, **kwargs):
        queryset = kwargs.pop('queryset', None)
        super().__init__(*args, **kwargs)
        self.fields['layer'].queryset = queryset





from geomanager.admin.base import BaseModelAdmin, ModelAdminCanHide
from geomanager.models import Metadata


class MetadataModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = Metadata
    exclude_from_explorer = True
    menu_icon = 'info-circle'



import json

from adminboundarymanager.models import AdminBoundarySettings
from django.contrib.gis.geos import GEOSGeometry, MultiPolygon
from django.core.exceptions import ObjectDoesNotExist
from django.db import connection
from django.http import HttpResponse, JsonResponse
from django.shortcuts import render
from django.urls import reverse
from django.utils.translation import gettext as _
from django.views import View
from wagtail.admin.views.generic import (
    IndexView,
    CreateView,
    EditView,
    DeleteView
)

from geomanager.forms.boundary import AdditionalBoundaryDataAddForm, AdditionalBoundaryDataEditForm
from geomanager.models import AdditionalMapBoundaryData, Geostore
from geomanager.serializers.geostore import GeostoreSerializer


def boundary_landing_view(request, ):
    links = []
    try:
        settings_url = reverse(
            "wagtailsettings:edit",
            args=[AdminBoundarySettings._meta.app_label, AdminBoundarySettings._meta.model_name, ],
        )
        links.append({
            "url": settings_url,
            "icon": "cog",
            "title": _("Boundary Settings"),
        })

        preview_url = reverse("adminboundarymanager_preview_boundary")

        links.append({
            "url": preview_url,
            "icon": "upload",
            "title": _("Country Boundary Loader"),
        })
    except Exception:
        pass

    links.append({
        "url": reverse("additional_boundary_index"),
        "icon": "layer-group",
        "title": _("Additional Map Boundaries"),
    })

    context = {
        "links": links,
    }

    return render(request, "geomanager/boundary/boundary_landing.html", context)


class AdditionalBoundaryIndexView(IndexView):
    model = AdditionalMapBoundaryData
    add_url_name = "additional_boundary_create"
    edit_url_name = "additional_boundary_edit"
    delete_url_name = "additional_boundary_delete"


class AdditionalBoundaryCreateView(CreateView):
    model = AdditionalMapBoundaryData
    form_class = AdditionalBoundaryDataAddForm
    index_url_name = "additional_boundary_index"
    add_url_name = "additional_boundary_create"
    edit_url_name = "additional_boundary_edit"
    success_message = _("Boundary dataset '%(object)s' created.")


class AdditionalBoundaryEditView(EditView):
    model = AdditionalMapBoundaryData
    form_class = AdditionalBoundaryDataEditForm
    edit_url_name = "additional_boundary_edit"
    index_url_name = "additional_boundary_index"


class AdditionalBoundaryDeleteView(DeleteView):
    model = AdditionalMapBoundaryData
    index_url_name = "additional_boundary_index"
    delete_url_name = "additional_boundary_delete"

    success_message = _("Boundary dataset '%(object)s' deleted.")


class AdditionalBoundaryVectorTileView(View):
    def get(self, request, table_name, z, x, y):
        try:
            vector_table = AdditionalMapBoundaryData.objects.get(table_name=table_name)
            # create list of columns to include in the vector tiles
            columns_sql = ", ".join(vector_table.columns) if vector_table.columns else "*"
        except ObjectDoesNotExist:
            return HttpResponse(f"Table matching 'table_name': {table_name} not found", status=404)

        sql = f"""WITH
                    bounds AS (
                      SELECT ST_TileEnvelope(%s, %s, %s) AS geom
                    ),
                    mvtgeom AS (
                      SELECT ST_AsMVTGeom(ST_Transform(t.geom, 3857), bounds.geom) AS geom,
                        {columns_sql}
                      FROM {vector_table.full_table_name} t, bounds
                      WHERE ST_Intersects(ST_Transform(t.geom, 4326), ST_Transform(bounds.geom, 4326))
                    )
                    SELECT ST_AsMVT(mvtgeom, 'default') FROM mvtgeom;
                    """

        with connection.cursor() as cursor:
            try:
                cursor.execute(sql, (z, x, y))
                tile = cursor.fetchone()[0]
                if not tile:
                    return HttpResponse("Tile not found", status=404)
                return HttpResponse(tile, content_type="application/x-protobuf")
            except Exception as e:
                return HttpResponse(f"Error while fetching tile: {e}", status=500)


def get_boundary_data_feature_by_id(request, table_name, gid):
    identifier = f"{table_name}-{gid}"

    try:
        vector_table = AdditionalMapBoundaryData.objects.get(table_name=table_name)
    except ObjectDoesNotExist:
        return JsonResponse({"message": f"Table with name: '{table_name}' does not exist"}, status=404)

    geostore = Geostore.objects.filter(iso=identifier).first()
    if geostore:
        res_data = GeostoreSerializer(geostore).data
        return JsonResponse(res_data)

    with connection.cursor() as cursor:
        query = f"""
            SELECT json_build_object(
                'type', 'FeatureCollection', 
                'features', json_agg(feature)
            ) FROM (
                SELECT json_build_object(
                    'type', 'Feature', 
                    'geometry', ST_AsGeoJSON(geom)::json, 
                    'properties', to_jsonb(inputs) - 'geom'
                ) AS feature 
                FROM (
                    SELECT gid, geom
                    FROM {vector_table.full_table_name}
                    WHERE gid={gid}
                ) AS inputs
            ) AS features;
        """

        cursor.execute(query)
        data = cursor.fetchone()[0]

    if data.get("features") is None:
        return JsonResponse({"message": "Not Found"}, status=404)

    feature = data.get("features")[0]
    geom = GEOSGeometry(json.dumps(feature.get("geometry")))

    if geom.geom_type == "Polygon":
        geom = MultiPolygon(geom)

    # create a new Geostore object and save it to the database
    geostore = Geostore(geom=geom, iso=identifier)
    geostore.save()

    res_data = GeostoreSerializer(geostore).data

    return JsonResponse(res_data)



# Generated by Django 4.1.10 on 2023-11-30 14:01

from django.db import migrations, models
import wagtail.blocks
import wagtail.fields
import wagtail.images.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0037_dataset_enable_all_multi_layers_on_add'),
    ]

    operations = [
        migrations.AddField(
            model_name='rasterfilelayer',
            name='legend',
            field=wagtail.fields.StreamField([('legend', wagtail.blocks.StructBlock([('type', wagtail.blocks.ChoiceBlock(choices=[('basic', 'Basic'), ('gradient', 'Gradient'), ('choropleth', 'Choropleth')], label='Legend Type')), ('items', wagtail.blocks.ListBlock(wagtail.blocks.StructBlock([('value', wagtail.blocks.CharBlock(help_text="Can be a number or text e.g '10' or '10-20' or 'Vegetation'", label='value')), ('color', wagtail.blocks.CharBlock(help_text='Color value e.g rgb(73,73,73) or #494949', label='color'))]), label='Legend Items', min_num=1))], label='Custom Legend')), ('legend_image', wagtail.images.blocks.ImageChooserBlock(label='Custom Image'))], blank=True, null=True, use_json_field=True, verbose_name='Legend'),
        ),
        migrations.AddField(
            model_name='rasterfilelayer',
            name='use_custom_legend',
            field=models.BooleanField(default=False, verbose_name='Use custom legend'),
        ),
    ]



# Generated by Django 4.1.10 on 2023-11-09 07:53

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0029_rasterfilelayer_auto_ingest_from_directory'),
    ]

    operations = [
        migrations.DeleteModel(
            name='StationSettings',
        ),
    ]



# Generated by Django 4.1.10 on 2024-02-26 12:28

from django.db import migrations, models
import geomanager.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0042_alter_dataset_options_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='AdditionalMapBoundaryData',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True)),
                ('table_name', models.CharField(max_length=256, unique=True)),
                ('properties', models.JSONField()),
                ('geometry_type', models.CharField(max_length=100)),
                ('bounds', geomanager.fields.ListField(max_length=256)),
            ],
            options={
                'verbose_name': 'Additional Map Boundary Layer',
                'verbose_name_plural': 'Additional Map Boundary Layers',
            },
        ),
    ]



# Generated by Django 4.2.10 on 2024-04-23 07:13

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
        ('geomanager', '0046_colorvalue_show_on_legend_rasterstyle_legend_type'),
    ]

    operations = [
        migrations.AddField(
            model_name='geomanagersettings',
            name='map_disclaimer_page',
            field=models.ForeignKey(blank=True, help_text='MapViewer Map disclaimer page for more details', null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.page', verbose_name='Map Disclaimer Page'),
        ),
        migrations.AddField(
            model_name='geomanagersettings',
            name='map_disclaimer_text',
            field=models.CharField(blank=True, default='The boundaries and names shown and the designations used on this map do not imply the expression of any opinion whatsoever concerning the legal status of any country, territory, city or area or of its authorities, or concerning the delimitation of its frontiers or boundaries', max_length=350, null=True, verbose_name='Map Disclaimer Text'),
        ),
        migrations.AlterField(
            model_name='rasterfilelayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)'), ('dekadal', 'Dekadal - (E.g Jan 2023 - D1 1-10th)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('yyyy', 'Year - (E.g 2023)')], default='yyyy-MM-dd HH:mm', max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
        migrations.AlterField(
            model_name='rasterstyle',
            name='legend_type',
            field=models.CharField(choices=[('basic', 'Basic'), ('choropleth', 'Choropleth Horizontal'), ('choropleth_vertical', 'Choropleth Vertical'), ('gradient', 'Gradient Horizontal'), ('gradient_vertical', 'Gradient Vertical')], default='choropleth_vertical', max_length=100, verbose_name='Legend Type'),
        ),
        migrations.AlterField(
            model_name='rastertilelayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)'), ('dekadal', 'Dekadal - (E.g Jan 2023 - D1 1-10th)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('yyyy', 'Year - (E.g 2023)')], max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
        migrations.AlterField(
            model_name='vectortilelayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)'), ('dekadal', 'Dekadal - (E.g Jan 2023 - D1 1-10th)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('yyyy', 'Year - (E.g 2023)')], max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
        migrations.AlterField(
            model_name='wmslayer',
            name='date_format',
            field=models.CharField(blank=True, choices=[('yyyy-MM-dd HH:mm', 'Hour minute:second - (E.g 2023-01-01 00:00)'), ('yyyy-MM-dd', 'Day - (E.g 2023-01-01)'), ('pentadal', 'Pentadal - (E.g Jan 2023 - P1 1-5th)'), ('dekadal', 'Dekadal - (E.g Jan 2023 - D1 1-10th)'), ('yyyy-MM', 'Month number - (E.g 2023-01)'), ('MMMM yyyy', 'Month name - (E.g January 2023)'), ('yyyy', 'Year - (E.g 2023)')], max_length=100, null=True, verbose_name='Display Format for DateTime Selector'),
        ),
    ]



import logging

from django.core.management.base import BaseCommand
from django.db import connection
from django.db.utils import OperationalError

from geomanager.settings import geomanager_settings

logger = logging.getLogger(__name__)


def ensure_pg_service_schema_exists(schema):
    if schema is not None:
        with connection.cursor() as cursor:
            try:
                schema_sql = f"""DO
                                    $do$
                                    BEGIN
                                        CREATE SCHEMA IF NOT EXISTS {schema};
                                    END
                                    $do$;"""
                cursor.execute(schema_sql)
            except OperationalError:
                # If the schema already exists, do nothing
                pass


class Command(BaseCommand):
    help = "Initialize GeoManager Vector Database Schema"

    def handle(self, *args, **options):
        # Create GeoManager vector database schema
        logger.info('[GEOMANAGER]: Creating GeoManager vector database schema...')

        vector_db_schema = geomanager_settings.get("vector_db_schema")

        ensure_pg_service_schema_exists(vector_db_schema)

        logger.info('[GEOMANAGER]: GeoManager vector database schema created.')



# Generated by Django 4.2.10 on 2024-02-28 15:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0044_alter_additionalmapboundarydata_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='additionalmapboundarydata',
            name='active',
            field=models.BooleanField(default=True, verbose_name='Active'),
        ),
        migrations.AlterField(
            model_name='additionalmapboundarydata',
            name='name',
            field=models.CharField(max_length=255, unique=True, verbose_name='Dataset Name'),
        ),
    ]



# Generated by Django 4.1.10 on 2023-07-10 19:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0083_workflowcontenttype'),
        ('geomanager', '0005_delete_countryboundary_and_more'),
    ]

    operations = [
        migrations.CreateModel(
            name='StationSettings',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('columns', models.JSONField(blank=True, null=True)),
                ('id_field', models.CharField(default='station_id', max_length=100)),
                ('site', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, to='wagtailcore.site')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import json

from django.shortcuts import get_object_or_404, render
from django.utils.translation import gettext as _
from wagtail.admin.auth import (
    user_passes_test,
    user_has_any_page_permission
)
from wagtail_modeladmin.helpers import AdminURLHelper

from geomanager.models import (
    Dataset, Category
)
from geomanager.models.raster_tile import RasterTileLayer
from geomanager.serializers.raster_tile import RasterTileLayerSerializer
from geomanager.utils import UUIDEncoder


@user_passes_test(user_has_any_page_permission)
def preview_raster_tile_layers(request, dataset_id, layer_id=None):
    dataset = get_object_or_404(Dataset, pk=dataset_id)

    category_admin_helper = AdminURLHelper(Category)
    categories_url = category_admin_helper.get_action_url("index")

    dataset_admin_helper = AdminURLHelper(Dataset)
    dataset_list_url = dataset_admin_helper.get_action_url("index") + f"?id={str(dataset_id)}"

    raster_tile_layer_admin_helper = AdminURLHelper(RasterTileLayer)
    raster_tile_layer_list_url = raster_tile_layer_admin_helper.get_action_url("index")
    raster_tile_layer_list_url = raster_tile_layer_list_url + f"?dataset__id__exact={str(dataset_id)}"

    layer = None
    if layer_id:
        layer = RasterTileLayer.objects.get(pk=layer_id)

    dataset_layers = RasterTileLayerSerializer(dataset.raster_tile_layers, many=True).data

    navigation_items = [
        {"url": categories_url, "label": Category._meta.verbose_name_plural},
        {"url": dataset_list_url, "label": Dataset._meta.verbose_name_plural},
        {"url": raster_tile_layer_list_url, "label": RasterTileLayer._meta.verbose_name_plural},
        {"url": "#", "label": _("Preview")},
    ]

    context = {
        "dataset": dataset,
        "selected_layer": layer,
        "datasets_index_url": dataset_list_url,
        "raster_tile_layer_list_url": raster_tile_layer_list_url,
        "dataset_layers": json.dumps(dataset_layers, cls=UUIDEncoder),
        "navigation_items": navigation_items,
    }

    return render(request, 'geomanager/raster_tile/raster_tile_layer_preview.html', context)



from django.urls import path

from geomanager.views import (
    boundary_landing_view,
    AdditionalBoundaryIndexView,
    AdditionalBoundaryCreateView, AdditionalBoundaryEditView, AdditionalBoundaryDeleteView
)

urls = [
    path('boundary-view/', boundary_landing_view, name='boundary_landing'),
    path('additional-boundary', AdditionalBoundaryIndexView.as_view(), name='additional_boundary_index'),
    path('additional-boundary/add', AdditionalBoundaryCreateView.as_view(), name='additional_boundary_create'),
    path('additional-boundary/edit/<int:pk>', AdditionalBoundaryEditView.as_view(),
         name='additional_boundary_edit'),
    path('additional-boundary/delete/<int:pk>', AdditionalBoundaryDeleteView.as_view(),
         name='additional_boundary_delete'),
]



from geomanager.constants import MAPBOX_GL_STYLE_SPEC


def get_vector_render_layers(render_layers_stream_field, tiled=False):
    render_layers = []

    optional_keys = ["filter", "maxzoom", "minzoom"]

    for layer in render_layers_stream_field:
        data = layer.block.get_api_representation(layer.value)

        render_layer_type = layer.block_type
        if render_layer_type == "icon" or render_layer_type == "text":
            render_layer_type = "symbol"

        data.update({"type": render_layer_type})

        # set source layer
        source_layer = "default"
        if data.get("source_layer"):
            source_layer = data.get("source_layer")
            del data["source_layer"]

        data.update({"source-layer": source_layer})

        # remove optional keys if they do not have any value
        for key in optional_keys:
            if not data.get(key):
                data.pop(key, None)

        paint_defaults = MAPBOX_GL_STYLE_SPEC.get("PAINT_DEFAULTS", {})
        layout_defaults = MAPBOX_GL_STYLE_SPEC.get("LAYOUT_DEFAULTS", {})

        paint = {}
        for key, value in data.get("paint", {}).items():
            default_spec_value = paint_defaults.get(key)
            #  if is equal to default value, no need to include it
            if default_spec_value == value:
                continue
            js_key = key.replace("_", "-")
            paint.update({js_key: value})

        layout = {}
        for key, value in data.get("layout", {}).items():
            default_spec_value = layout_defaults.get(key)
            #  if is equal to default value, no need to include it
            if default_spec_value == value:
                continue
            js_key = key.replace("_", "-")
            layout.update({js_key: value})

        if bool(paint):
            data.update({"paint": paint})
        else:
            # nothing for paint. Just delete it
            data.pop("paint", None)

        if bool(layout):
            data.update({"layout": layout})
        else:
            # nothing for layout. Just delete it
            data.pop("layout", None)

        data.update({
            "metadata": {
                "position": "top",
            }
        })

        render_layers.append(data)

    return render_layers



from django.utils.translation import gettext_lazy as _

from geomanager.admin.base import BaseModelAdmin, ModelAdminCanHide
from geomanager.models import MBTSource


class MBTSourceModelAdmin(BaseModelAdmin, ModelAdminCanHide):
    model = MBTSource
    menu_label = _("Basemap Sources")
    menu_icon = "globe"
    form_view_extra_js = ["geomanager/js/mbt_source_extra.js"]



# Generated by Django 4.1.10 on 2023-10-25 11:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0025_alter_areaofinterest_options_alter_category_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='layerrasterfile',
            name='raster_metadata',
            field=models.JSONField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='colorvalue',
            name='label',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='Optional Label'),
        ),
    ]



from django import forms
from django.db import models
from django.utils.translation import gettext_lazy as _
from modelcluster.models import ClusterableModel
from wagtail import blocks
from wagtail.admin.panels import (
    FieldPanel,
    TabbedInterface,
    ObjectList, MultiFieldPanel
)
from wagtail.api.v2.utils import get_full_url
from wagtail.contrib.settings.models import BaseSiteSetting
from wagtail.contrib.settings.registry import register_setting
from wagtail.fields import StreamField
from wagtail.images.blocks import ImageChooserBlock
from wagtail.models import Page

from geomanager.blocks import NavigationItemsBlock
from .tile_gl import MBTSource
from ..fields import ListField

DEFAULT_RASTER_MAX_UPLOAD_SIZE_MB = 100


@register_setting
class GeomanagerSettings(BaseSiteSetting, ClusterableModel):
    max_upload_size_mb = models.IntegerField(default=DEFAULT_RASTER_MAX_UPLOAD_SIZE_MB,
                                             verbose_name=_("Maximum upload size in MegaBytes"),
                                             help_text=_(
                                                 "Maximum raster file size that can be uploaded in MegaBytes. "
                                                 "Default is 100Mbs."))
    crop_raster_to_country = models.BooleanField(default=True, verbose_name=_("Crop raster to country"),
                                                 help_text=_("Crop the uploaded raster file to the country boundaries"))

    tile_gl_fonts_url = models.URLField(max_length=256,
                                        default="https://fonts.openmaptiles.org/{fontstack}/{range}.pbf",
                                        verbose_name=_("GL Styles Font Url"),
                                        help_text=_("GL Styles Font Url"))
    tile_gl_source = models.ForeignKey(MBTSource, blank=True, null=True, on_delete=models.SET_NULL,
                                       verbose_name=_("Open Map Tiles Source"))
    logo = models.ForeignKey(
        'wagtailimages.Image',
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Logo")
    )
    logo_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Logo page"),
        help_text=_("Internal page to navigate to on clicking the logo")
    )
    logo_external_link = models.URLField(max_length=255, null=True, blank=True,
                                         verbose_name=_("Logo external link"),
                                         help_text=_("Used if internal logo page not provided"))
    terms_of_service_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Terms of Service Page"),
        help_text=_("MapViewer Terms of Service page")
    )
    privacy_policy_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Privacy Policy Page"),
        help_text=_("MapViewer Privacy Policy Page")
    )
    contact_us_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Contact Us Page")
    )

    navigation = StreamField([
        ('menu_items', blocks.ListBlock(NavigationItemsBlock(max_num=8))),
    ], block_counts={
        'menu_items': {'max_num': 1},
    }, use_json_field=True, null=True, blank=True)

    map_disclaimer_text = models.CharField(max_length=350, blank=True, null=True,
                                           default="The boundaries and names shown and the designations used on this "
                                                   "map do not imply the expression of any opinion whatsoever "
                                                   "concerning the legal status of any country, territory, city or "
                                                   "area or of its authorities, or concerning the delimitation of its "
                                                   "frontiers or boundaries",
                                           verbose_name=_("Map Disclaimer Text"),
                                           help_text=_("MapViewer Map disclaimer text. Maximum of 350 characters. "
                                                       "Add more details in the Map Disclaimer Page and link it below"))
    map_disclaimer_page = models.ForeignKey(
        Page,
        null=True,
        blank=True,
        on_delete=models.SET_NULL,
        related_name='+',
        verbose_name=_("Map Disclaimer Page"),
        help_text=_("MapViewer Map disclaimer page for more details")
    )

    enable_my_account = models.BooleanField(default=False, verbose_name=_("Enable My Account"))
    allow_signups = models.BooleanField(default=False, verbose_name=_("Allow user signups"))

    base_maps = StreamField([
        ('basemap', blocks.StructBlock([
            ('label', blocks.CharBlock(label=_("label"))),
            ('backgroundColor', blocks.CharBlock(label=_("background color"))),
            ('image', ImageChooserBlock(required=False, label=_("image"))),
            ('basemapGroup', blocks.CharBlock(label=_("basemap group"))),
            ('labelsGroup', blocks.CharBlock(label=_("labels group"))),
            ('url', blocks.URLBlock(required=False, label=_("url"))),
            ('default', blocks.BooleanBlock(required=False, label=_("default"), help_text=_("Is default style ?"))),
        ]))
    ], use_json_field=True, null=True, blank=True)

    class Meta:
        verbose_name = _("Geomanager Settings")

    edit_handler = TabbedInterface([
        ObjectList([
            FieldPanel("max_upload_size_mb"),
            FieldPanel("crop_raster_to_country"),
        ], heading=_("Upload Settings")),
        ObjectList([
            FieldPanel("tile_gl_fonts_url"),
            FieldPanel("tile_gl_source"),
            FieldPanel("base_maps"),
        ], heading=_("Basemap TileServer Settings")),
        ObjectList([
            FieldPanel("logo"),
            FieldPanel("logo_page"),
            FieldPanel("logo_external_link"),
            FieldPanel("terms_of_service_page"),
            FieldPanel("privacy_policy_page"),
            FieldPanel("contact_us_page"),
            FieldPanel("navigation"),

            MultiFieldPanel([
                FieldPanel("map_disclaimer_text", widget=forms.Textarea(attrs={'rows': 3})),
                FieldPanel("map_disclaimer_page")
            ], heading=_("Map Disclaimer Settings")),

        ], heading=_("Navigation Settings")),
        ObjectList([
            FieldPanel("enable_my_account"),
            FieldPanel("allow_signups"),
        ], heading=_("My Account Settings")),
    ])

    @property
    def max_upload_size_bytes(self):
        return self.max_upload_size_mb * 1024 * 1024

    @property
    def logo_link(self):
        if self.logo_page:
            return get_full_url(None, self.logo_page.url)

        if self.logo_external_link:
            return self.logo_external_link

        return "/"



import re

from django.core.exceptions import ValidationError
from django.utils.translation import gettext_lazy as _

valid_directory_names_pattern = re.compile(r'^[a-zA-Z0-9_/-]+$')


def validate_directory_name(value):
    if value.startswith('/'):
        raise ValidationError(_("The directory name can not start with '/'. "), code='invalid', )
    if value.endswith('/'):
        raise ValidationError(_("The directory name can not end with '/'. "), code='invalid', )

    if not valid_directory_names_pattern.match(value):
        raise ValidationError(
            _("The directory name can only contain alphanumeric characters, underscores, and hyphens."),
            code='invalid', )

    return value



from wagtail.admin.panels import FieldPanel


class ReadOnlyFieldPanel(FieldPanel):
    class BoundPanel(FieldPanel.BoundPanel):
        def __init__(self, **kwargs):
            super().__init__(**kwargs)

            # swap html disabled --> readonly
            self.bound_field.field.disabled = True
            self.bound_field.field.widget.attrs['readonly'] = True



from django import forms
from django.forms import ModelForm

from geomanager.models.aoi import AreaOfInterest


class AoiForm(ModelForm):
    class Meta:
        model = AreaOfInterest
        fields = "__all__"



import uuid

from django.contrib.gis.db import models
from django_extensions.db.models import TimeStampedModel


class Geostore(TimeStampedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    iso = models.CharField(max_length=100, blank=True, null=True)
    id1 = models.CharField(max_length=100, blank=True, null=True)
    id2 = models.CharField(max_length=100, blank=True, null=True)
    id3 = models.CharField(max_length=100, blank=True, null=True)

    name_0 = models.CharField(max_length=100, blank=True, null=True)
    name_1 = models.CharField(max_length=100, blank=True, null=True)
    name_2 = models.CharField(max_length=100, blank=True, null=True)
    name_3 = models.CharField(max_length=100, blank=True, null=True)
    name_4 = models.CharField(max_length=100, blank=True, null=True)

    geom = models.MultiPolygonField(srid=4326)

    def __str__(self):
        return self.id.hex

    @property
    def bbox(self):
        min_x, min_y, max_x, max_y = self.geom.envelope.extent
        bbox = [min_x, min_y, max_x, max_y]
        return bbox

    @property
    def info(self):

        info = {}

        if self.iso:
            info.update({"iso": self.iso, "name": self.name_0})

        if self.id1:
            info.update({"id1": self.id1, "name": self.name_1})

        if self.id2:
            info.update({"id2": self.id2, "name": self.name_2})

        if self.id3:
            info.update({"id3": self.id3, "name": self.id3})

        return info



import logging

from wagtailcache.cache import clear_cache

from .aoi import *
from .boundary import *
from .core import *
from .geomanager_settings import *
from .geostore import *
from .profile import *
from .raster_file import *
from .raster_tile import *
from .tile_gl import *
from .vector_file import *
from .vector_tile import *
from .wms import *
from .raster_style import *

logger = logging.getLogger(__name__)


# clear wagtail cache on saving the following models
@receiver(post_save, sender=Category)
@receiver(post_save, sender=SubCategory)
@receiver(post_save, sender=Dataset)
@receiver(post_save, sender=Metadata)
@receiver(post_save, sender=GeomanagerSettings)
@receiver(post_save, sender=RasterFileLayer)
@receiver(post_save, sender=LayerRasterFile)
@receiver(post_save, sender=RasterStyle)
@receiver(post_save, sender=WmsLayer)
@receiver(post_save, sender=MBTSource)
@receiver(post_save, sender=VectorFileLayer)
@receiver(post_save, sender=PgVectorTable)
@receiver(post_save, sender=AdditionalMapBoundaryData)
def handle_clear_wagtail_cache(sender, **kwargs):
    logging.info("[WAGTAIL_CACHE]: Clearing cache")
    clear_cache()



import uuid

from django.contrib.admin.utils import quote
from django.contrib.gis.db import models
from django.core.files.base import ContentFile
from django.db.models.signals import pre_delete
from django.dispatch import receiver
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel
from wagtail.api.v2.utils import get_full_url
from wagtail.fields import StreamField
from wagtail.images.blocks import ImageChooserBlock

from geomanager.blocks import (
    InlineLegendBlock,
    FillVectorLayerBlock,
    LineVectorLayerBlock,
    CircleVectorLayerBlock,
    IconVectorLayerBlock,
    TextVectorLayerBlock,
    InlineIconLegendBlock
)
from geomanager.fields import ListField
from geomanager.helpers import get_vector_layer_files_url
from geomanager.models.core import Dataset, BaseLayer
from geomanager.models.vector_tile import VectorTileLayer
from geomanager.panels import ReadOnlyFieldPanel
from geomanager.utils.svg import rasterize_svg_to_png
from geomanager.utils.tiles import get_vector_render_layers
from geomanager.utils.vector_utils import drop_vector_table


class VectorFileLayer(TimeStampedModel, ClusterableModel, BaseLayer):
    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name="vector_file_layers",
                                verbose_name=_("dataset"))
    render_layers = StreamField([
        ("fill", FillVectorLayerBlock(label=_("Polygon Layer"))),
        ("line", LineVectorLayerBlock(label=_("Line Layer"))),
        ("circle", CircleVectorLayerBlock(label=_("Point Layer"))),
        ("icon", IconVectorLayerBlock(label=_("Icon Layer"))),
        ("text", TextVectorLayerBlock(label=_("Text Label Layer"))),
    ], use_json_field=True, null=True, blank=True, min_num=1, verbose_name=_("Render Layers"))

    legend = StreamField([
        ('legend', InlineLegendBlock(label=_("Legend")),),
        ('legend_image', ImageChooserBlock(label=_("Legend Image")),),
        ('legend_icon', InlineIconLegendBlock(label=_("Legend Icon")),)
    ], use_json_field=True, null=True, blank=True, max_num=1, verbose_name=_("Legend"), )

    class Meta:
        verbose_name = _("Vector File Layer")
        verbose_name_plural = _("Vector File Layers")
        ordering = ['order']

    def __str__(self):
        return self.title

    def get_uploads_list_url(self):
        url = get_vector_layer_files_url(self.pk)
        return url

    @property
    def has_data_table(self):
        return self.vector_tables.all().exists()

    @property
    def upload_url(self):
        upload_url = reverse(
            f"geomanager_dataset_layer_upload_vector",
            args=[quote(self.dataset.pk), quote(self.pk)],
        )
        return upload_url

    @property
    def preview_url(self):
        preview_url = reverse(
            f"geomanager_preview_vector_layer",
            args=[quote(self.dataset.pk), quote(self.pk)],
        )
        return preview_url

    def layer_config(self, request=None):
        base_tiles_url = reverse("vector_tiles", args=(0, 0, 0))
        base_tiles_url = base_tiles_url.replace("/0/0/0", r"/{z}/{x}/{y}")

        if request:
            base_tiles_url = get_full_url(request, base_tiles_url)

        tile_url = f"{base_tiles_url}?table_name={{table_name}}"

        if self.dataset.can_clip:
            tile_url = tile_url + "&geostore_id={geostore_id}"

        layer_config = {
            "type": "vector",
            "source": {
                "type": "vector",
                "tiles": [tile_url]
            }
        }

        render_layers = get_vector_render_layers(self.render_layers)

        layer_config.update({"render": {"layers": render_layers}})

        return layer_config

    @property
    def params(self):
        recent = self.vector_tables.first()

        params = {
            "table_name": recent.table_name if recent else ""
        }

        if self.dataset.can_clip:
            params.update({"geostore_id": ""})

        return params

    @property
    def param_selector_config(self):
        config = {
            "key": "table_name",
            "required": True,
            "sentence": "{selector}",
        }

        return [config]

    def get_legend_config(self, request=None):
        # default config
        config = {
            "type": "basic",
            "items": []
        }

        legend_block = self.legend

        # only one legend block entry is expected
        if legend_block:
            legend_block = legend_block[0]

        if legend_block:

            if legend_block.block_type == "legend_image":
                image_url = legend_block.value.file.url
                if request:
                    image_url = get_full_url(request, image_url)
                config.update({"type": "image", "imageUrl": image_url})
                return config

            data = legend_block.block.get_api_representation(legend_block.value)

            if legend_block.block_type == "legend_icon":
                for item in data.get("items", []):
                    config["items"].append({
                        "icon": item.get("icon_image"),
                        "name": item.get("icon_label"),
                        "color": item.get("icon_color"),
                        "iconSource": "sprite",
                    })
                return config

            config.update({"type": data.get("type", config.get("type"))})

            for item in data.get("items", []):
                config["items"].append({
                    "name": item.get("value"),
                    "color": item.get("color")
                })

        return config

    @property
    def interaction_config(self):
        config = {}

        # TODO: What happens if we have multiple tables ?
        vector_table = self.vector_tables.first()

        if vector_table and vector_table.properties:
            for column in vector_table.properties:
                if column.get("popup"):
                    if not config.get("output"):
                        config["output"] = []
                    name = column.get("name")
                    label = column.get("label")
                    config["output"].append({"column": name, "property": label or name}, )

        return config

    def save(self, *args, **kwargs):
        # remove existing icons for this layer.
        # TODO: Find efficient way to update exising icons, while deleting absolute ones
        VectorLayerIcon.objects.filter(layer=self).delete()

        for render_layer in self.render_layers:
            if render_layer.block_type == "icon":
                icon_image = render_layer.value.get("layout").get("icon_image")
                icon_color = render_layer.value.get("paint").get("icon_color")
                png_bytes = rasterize_svg_to_png(icon_image, fill_color=icon_color)

                if png_bytes:
                    layer_icon = VectorLayerIcon(name=icon_image, color=icon_color)
                    layer_icon.file = ContentFile(png_bytes.getvalue(), f"{icon_image}.{icon_color}.png")
                    self.icons.add(layer_icon)

        super().save(*args, **kwargs)


class VectorLayerIcon(models.Model):
    layer = ParentalKey(VectorFileLayer, on_delete=models.CASCADE, related_name="icons")
    name = models.CharField(max_length=255)
    color = models.CharField(max_length=100, null=True)
    file = models.FileField(upload_to="vector_icons/")

    class Meta:
        verbose_name = _("Vector Layer Icon")
        verbose_name_plural = _("Vector Layer Icons")

    def __str__(self):
        return f"{self.layer.title}-{self.name}-{self.color}"


class VectorUpload(TimeStampedModel):
    dataset = models.ForeignKey(Dataset, blank=True, null=True, on_delete=models.SET_NULL)
    file = models.FileField(upload_to="vector_uploads")
    vector_metadata = models.JSONField(blank=True, null=True)

    class Meta:
        verbose_name = _("Vector Upload")
        verbose_name_plural = _("Vector Uploads")

    panels = [
        FieldPanel("dataset"),
        FieldPanel("file"),
    ]

    def __str__(self):
        return f"{self.dataset} - {self.created}"


class PgVectorTable(TimeStampedModel):
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    layer = models.ForeignKey(VectorFileLayer, on_delete=models.CASCADE, related_name="vector_tables")
    table_name = models.CharField(max_length=256, unique=True)
    full_table_name = models.CharField(max_length=256)
    description = models.TextField(blank=True, null=True)
    time = models.DateTimeField(help_text=_("time for the dataset"))

    properties = models.JSONField()
    geometry_type = models.CharField(max_length=100)
    bounds = ListField(max_length=256)

    panels = [
        ReadOnlyFieldPanel("layer"),
        ReadOnlyFieldPanel("table_name"),
        ReadOnlyFieldPanel("full_table_name"),
        ReadOnlyFieldPanel("description"),
        ReadOnlyFieldPanel("time"),
        ReadOnlyFieldPanel("properties"),
        ReadOnlyFieldPanel("geometry_type"),
        ReadOnlyFieldPanel("bounds"),
    ]

    class Meta:
        ordering = ["-time"]
        unique_together = ('layer', 'time')
        verbose_name = _("Vector Upload")
        verbose_name_plural = _("Vector Uploads")

    def __str__(self):
        return f"{self.layer}  - {self.geometry_type}"

    @property
    def columns(self):
        if not self.properties:
            return []
        if self.properties:
            return [c.get("name") for c in self.properties]


@receiver(pre_delete, sender=PgVectorTable)
def drop_pg_vector_table(sender, instance, **kwargs):
    if instance.full_table_name:
        drop_vector_table(instance.full_table_name)


def get_legend_icons():
    vector_file_layers = VectorFileLayer.objects.all()
    vector_tile_layers = VectorTileLayer.objects.all()
    icons = []

    for layer in [*vector_file_layers, *vector_tile_layers]:
        if layer.legend:
            legend_block = layer.legend[0]
            if legend_block.block_type == "legend_icon":
                items = legend_block.value.get("items")
                for item in items:
                    icons.append(item.get("icon_image"))
    return icons



from django.db import models
from django.utils.translation import gettext_lazy as _
from django_extensions.db.models import TimeStampedModel
from modelcluster.fields import ParentalKey
from modelcluster.models import ClusterableModel
from wagtail.admin.panels import FieldPanel, FieldRowPanel, MultiFieldPanel, InlinePanel
from wagtail.api.v2.utils import get_full_url
from wagtail.fields import StreamField
from wagtail.images.blocks import ImageChooserBlock
from wagtail.images.models import Image
from wagtail.models import Orderable

from geomanager.blocks import (
    InlineLegendBlock,
    LayerMoreInfoBlock,
    QueryParamSelectableBlock
)
from geomanager.models.core import Dataset, BaseLayer
from geomanager.utils import DATE_FORMAT_CHOICES


class WmsLayer(TimeStampedModel, ClusterableModel, BaseLayer):
    OUTPUT_FORMATS = (
        ("image/png", "PNG"),
        ("image/jpeg", "JPEG"),
        ("image/gif", "GIF"),
    )

    VERSION_CHOICES = (
        ("1.1.1", "1.1.1"),
        ("1.3.0", "1.3.0"),
    )

    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name="wms_layers", verbose_name=_("dataset"))
    request_time_from_capabilities = models.BooleanField(default=True, verbose_name=_("Request time from capabilities"),
                                                         help_text=_("Get list of available times for this layer"
                                                                     " from getCapabilities"), )
    base_url = models.CharField(max_length=500, verbose_name=_("base url for WMS"), )
    version = models.CharField(max_length=50, default="1.1.1", choices=VERSION_CHOICES, verbose_name=_("WMS Version"))
    width = models.IntegerField(default=256, verbose_name=_("Pixel Width"),
                                help_text=_("The size of the map image in pixels along the i axis"), )
    height = models.IntegerField(default=256, verbose_name=_("Pixel Height"),
                                 help_text=_("The size of the map image in pixels along the j axis"), )
    transparent = models.BooleanField(default=True, verbose_name=_("Transparency"),
                                      help_text=_("Ability of underlying maps to be visible or not"), )
    srs = models.CharField(max_length=50, default="EPSG:3857", verbose_name=_("Spatial Reference System"),
                           help_text=_("WMS Spatial Reference e.g EPSG:3857"), )
    format = models.CharField(max_length=50, default="image/png", choices=OUTPUT_FORMATS,
                              verbose_name=_("Output Format"),
                              help_text=_(
                                  "Allowed map formats are either “picture” formats or “graphic element” formats."), )
    wms_query_params_selectable = StreamField([
        ('param', QueryParamSelectableBlock(label=_("Query Parameter")))
    ], use_json_field=True, null=True, blank=True, verbose_name=_("WMS Query Params With selectable Options"),
        help_text=_(
            "This should provide a list of options that users can choose to change the query parameter of the url"))

    params_selectors_side_by_side = models.BooleanField(default=False,
                                                        verbose_name=_("Arrange Param Selectors side by side"))
    legend = StreamField([
        ('legend', InlineLegendBlock(label=_("Custom Legend")),),
        ('legend_image', ImageChooserBlock(label=_("Custom Image")),),
    ], use_json_field=True, null=True, blank=True, max_num=1, verbose_name=_("Legend"), )

    date_format = models.CharField(max_length=100, choices=DATE_FORMAT_CHOICES, blank=True, null=True,
                                   verbose_name=_("Display Format for DateTime Selector"))
    custom_get_capabilities_url = models.URLField(blank=True, null=True, verbose_name=_("Get Capabilities URL"),
                                                  help_text=_("Alternative URL for the GetCapabilities request. "
                                                              "Used when 'Request time from capabilities' "
                                                              "option is checked. Leave blank to use base url."))
    get_capabilities_layer_name = models.CharField(max_length=255, blank=True, null=True,
                                                   verbose_name=_("Get Capabilities Layer Name"),
                                                   help_text=_("Alternative layer name for the GetCapabilities request"
                                                               ))
    more_info = StreamField([
        ('more_info', LayerMoreInfoBlock(label=_("Info link")),),
    ], block_counts={
        'more_info': {'max_num': 1},
    }, use_json_field=True, null=True, blank=True, max_num=1, verbose_name=_("More Info"), )

    class Meta:
        verbose_name = _("WMS Layer")
        verbose_name_plural = _("WMS Layers")
        ordering = ['order']

    panels = [
        FieldPanel("dataset"),
        FieldPanel("title"),
        FieldPanel("default"),
        FieldPanel("date_format"),
        MultiFieldPanel([
            FieldPanel("base_url"),
            FieldPanel("version"),
            FieldRowPanel([
                FieldPanel("width", classname="col6"),
                FieldPanel("height", classname="col6"),
            ]),
            FieldPanel("transparent"),
            FieldPanel("srs"),
            FieldPanel("format"),
            InlinePanel("wms_request_layers", heading=_("WMS Request Layers"),
                        label=_("WMS Request Layer"), min_num=1),
            InlinePanel("wms_request_styles", heading=_("WMS Request Styles"),
                        label=_("WMS Request Style")),
            InlinePanel("wms_request_params", heading=_("WMS Request Additional Parameters"),
                        label=_("WMS Request Param")),
            FieldPanel("wms_query_params_selectable"),
        ], heading=_("WMS GetMap Configuration")),
        MultiFieldPanel([
            FieldPanel("request_time_from_capabilities"),
            FieldPanel("custom_get_capabilities_url"),
            FieldPanel("get_capabilities_layer_name"),
        ], heading=_("WMS GetCapabilities Configuration")),
        FieldPanel("params_selectors_side_by_side"),
        FieldPanel("legend"),
        FieldPanel("more_info"),
    ]

    def __str__(self):
        return self.title

    def get_selectable_params(self):
        params = {}
        if self.wms_query_params_selectable:
            for query_param in self.wms_query_params_selectable:
                data = query_param.block.get_api_representation(query_param.value)
                val = f"{data.get('name')}"
                params.update({val: data})
        return params

    def get_wms_params(self):
        params = {
            "SERVICE": "WMS",
            "VERSION": self.version,
            "REQUEST": "GetMap",
            "TRANSPARENT": self.transparent,
            "LAYERS": ",".join([layer.name for layer in self.wms_request_layers.all()]),
            "STYLES": ",".join([style.name for style in self.wms_request_styles.all()]),
            "BBOX": "{bbox-epsg-3857}",
            "WIDTH": self.width,
            "HEIGHT": self.height,
            "FORMAT": self.format,
        }

        if self.version == "1.3.0":
            params.update({"CRS": self.srs})
        else:
            params.update({"SRS": self.srs})

        extra_params = {param.name: param.value for param in self.wms_request_params.all()}
        params.update(**extra_params)

        return params

    @property
    def get_map_url(self):
        params = self.get_wms_params()
        selectable_params = self.get_selectable_params()

        for key, val in selectable_params.items():
            key_val = key
            if key.upper() in params:
                key_val = key.upper()
            params.update({key_val: f"{{{key}}}"})

        query_str = '&'.join([f"{key}={value}" for key, value in params.items()])
        request_url = f"{self.base_url}?{query_str}"

        return request_url

    def get_selectable_params_config(self):
        selectable_params = self.get_selectable_params()
        config = []

        for key, param_config in selectable_params.items():
            param_config = {
                "key": key,
                "required": True,
                "type": param_config.get("type"),
                "options": param_config.get("options"),
                "sentence": f"{param_config.get('label') or key} {{selector}}",
            }

            config.append(param_config)

        return config

    @property
    def get_capabilities_url(self):
        if self.request_time_from_capabilities:
            capabilities_url = self.base_url

            if self.custom_get_capabilities_url:
                capabilities_url = self.custom_get_capabilities_url

            params = {
                "SERVICE": "WMS",
                "VERSION": self.version,
                "REQUEST": "GetCapabilities",
            }
            query_str = '&'.join([f"{key}={value}" for key, value in params.items()])
            request_url = f"{capabilities_url}?{query_str}"
            return request_url
        return None

    @property
    def layer_config(self):
        wms_url = self.get_map_url
        if self.dataset.multi_temporal:
            wms_url = f"{wms_url}&time={{time}}"

        layer_config = {
            "type": "raster",
            "source": {
                "type": "raster",
                "tiles": [wms_url]
            }
        }

        return layer_config

    @property
    def params(self):
        params = {}
        if self.dataset.multi_temporal:
            params.update({"time": ""})

        selector_config = self.get_selectable_params_config()

        for selector_param in selector_config:
            default = None
            for option in selector_param.get("options"):
                if option.get("default"):
                    default = option.get("value")
                    break
            if not default:
                default = selector_param.get("options")[0].get("value")
            params.update({selector_param.get("key"): default})

        return params

    @property
    def time_parameter_selector_config(self):
        time_config = None
        if self.dataset.multi_temporal:
            time_config = {
                "key": "time",
                "required": True,
                "sentence": "{selector}",
                "type": "datetime",
                "availableDates": [],
            }

            if self.date_format:
                if self.date_format == "pentadal":
                    time_config.update({
                        "dateFormat": {"currentTime": "MMM yyyy", "asPeriod": "pentadal"},
                    })
                elif self.date_format == "dekadal":
                    time_config.update({
                        "dateFormat": {"currentTime": "MMM yyyy", "asPeriod": "dekadal"},
                    })
                else:
                    time_config.update({
                        "dateFormat": {"currentTime": self.date_format},
                    })
            else:
                time_config.update({
                    "dateFormat": {"currentTime": "yyyy-MM-dd HH:mm"},
                })

        return time_config

    @property
    def param_selector_config(self):
        config = []

        time_config = self.time_parameter_selector_config
        if time_config:
            config.append(time_config)

        selectable_params_config = self.get_selectable_params_config()
        config.extend(selectable_params_config)

        return config

    def get_legend_config(self, request):

        # default config
        config = {
            "type": "basic",
            "items": []
        }

        legend_block = self.legend

        # only one legend block entry is expected
        if legend_block:
            legend_block = legend_block[0]

        if legend_block:
            if isinstance(legend_block.value, Image):
                image_url = legend_block.value.file.url
                if request:
                    image_url = get_full_url(request, image_url)
                config.update({"type": "image", "imageUrl": image_url})
                return config

            data = legend_block.block.get_api_representation(legend_block.value)

            config.update({"type": data.get("type")})

            for item in data.get("items"):
                config["items"].append({
                    "name": item.get("value"),
                    "color": item.get("color")
                })

        return config

    @property
    def layer_name(self):
        return self.wms_request_layers.all()[0].name

    def get_analysis_config(self):
        analysis_config = []

        return analysis_config

    def get_more_info(self):
        more_info = self.more_info

        return {}


class WmsRequestLayer(Orderable):
    layer = ParentalKey(WmsLayer, on_delete=models.CASCADE, related_name="wms_request_layers",
                        verbose_name=_("WMS Request Layers"))
    name = models.CharField(max_length=250, null=False, blank=False,
                            verbose_name=_("name"),
                            help_text=_("WMS Layer is requested by using this "
                                        "name in the LAYERS parameter of a "
                                        "GetMap request."))

    class Meta:
        verbose_name = _("WMS Request Layer")
        verbose_name_plural = _("WMS Request Layers")


class WmsRequestStyle(Orderable):
    layer = ParentalKey(WmsLayer, on_delete=models.CASCADE,
                        related_name="wms_request_styles", verbose_name=_("WMS Request Styles"))
    name = models.CharField(max_length=250, null=False, blank=False,
                            verbose_name=_("name"),
                            help_text=_("The style's Name is used in the Map request STYLES parameter"))

    class Meta:
        verbose_name = _("WMS Request Style")
        verbose_name_plural = _("WMS Request Styles")


class WmsRequestParam(Orderable):
    layer = ParentalKey(WmsLayer, on_delete=models.CASCADE, related_name="wms_request_params",
                        verbose_name=_("WMS Requests Additional Parameters", ))
    name = models.CharField(max_length=250, null=False, blank=False, verbose_name=_("name"),
                            help_text=_("Name of the parameter"))
    value = models.CharField(max_length=250, null=False, blank=False, help_text=_("Value of the parameter"))

    class Meta:
        verbose_name = _("WMS Request Parameter")
        verbose_name_plural = _("WMS Request Parameters")



from rest_framework import viewsets
from rest_framework.permissions import IsAuthenticated
from rest_framework.renderers import JSONRenderer

from geomanager.models import AreaOfInterest
from geomanager.serializers.aoi import AoiSerializer


class AoiViewSet(viewsets.ModelViewSet):
    serializer_class = AoiSerializer
    queryset = AreaOfInterest.objects.all()
    permission_classes = [IsAuthenticated]
    renderer_classes = [JSONRenderer]

    def get_queryset(self):
        queryset = self.queryset.filter(user=self.request.user.id)
        return queryset

    def create(self, request, *args, **kwargs):
        request.data.update({"user": request.user.id})

        return super().create(request, *args, **kwargs)

    def update(self, request, *args, **kwargs):
        return super().update(request, *args, **kwargs)



from rest_framework import serializers

from geomanager.models import GeoManagerUserProfile


class GeoManagerUserProfileSerializer(serializers.ModelSerializer):
    email = serializers.SerializerMethodField()
    first_name = serializers.SerializerMethodField()
    last_name = serializers.SerializerMethodField()
    user_id = serializers.SerializerMethodField()
    full_name = serializers.SerializerMethodField()

    class Meta:
        model = GeoManagerUserProfile
        fields = ["user_id", "email", "first_name", "last_name", "full_name", "gender", "country", "city", "sector",
                  "organization", "organization_type", "scale_of_operations", "position", "how_do_you_use", "interests"]

    def get_user_id(self, obj):
        return obj.user.id

    def get_email(self, obj):
        return obj.user.email

    def get_first_name(self, obj):
        return obj.user.first_name

    def get_last_name(self, obj):
        return obj.user.last_name

    def get_last_name(self, obj):
        return obj.user.last_name

    def get_full_name(self, obj):
        if obj.user.first_name and obj.user.last_name:
            return f"{obj.user.first_name} {obj.user.last_name}"



import json

from adminboundarymanager.models import AdminBoundary, AdminBoundarySettings
from django.contrib.gis.geos import GEOSGeometry
from django.contrib.gis.geos import MultiPolygon
from django.utils.decorators import method_decorator
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import mixins
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.exceptions import NotFound
from rest_framework.renderers import JSONRenderer
from rest_framework.response import Response
from wagtailcache.cache import cache_page

from geomanager import serializers
from geomanager.decorators import revalidate_cache
from geomanager.models import Geostore
from geomanager.models.vector_file import PgVectorTable
from geomanager.serializers.geostore import GeostoreSerializer
from geomanager.serializers.vector_file import AdminBoundarySerializer


class VectorTableFileDetailViewSet(mixins.ListModelMixin, viewsets.GenericViewSet):
    renderer_classes = [JSONRenderer]
    queryset = PgVectorTable.objects.all()
    serializer_class = serializers.PgVectorTableSerializer
    filter_backends = [DjangoFilterBackend]
    filterset_fields = ["layer"]


class AdminBoundaryViewSet(viewsets.ViewSet):
    renderer_classes = [JSONRenderer]

    @action(detail=True, methods=['get'])
    @method_decorator(revalidate_cache)
    @method_decorator(cache_page)
    def get(self, request):
        countries = AdminBoundary.objects.filter(level=0)
        data = AdminBoundarySerializer(countries, many=True).data
        return Response(data)

    @action(detail=True, methods=['get'])
    @method_decorator(revalidate_cache)
    @method_decorator(cache_page)
    def get_regions(self, request, gid_0):
        countries = AdminBoundary.objects.filter(level=1, gid_0=gid_0)
        data = AdminBoundarySerializer(countries, many=True).data
        return Response(data)

    @action(detail=True, methods=['get'])
    @method_decorator(revalidate_cache)
    @method_decorator(cache_page)
    def get_sub_regions(self, request, gid_0, gid_1):
        countries = AdminBoundary.objects.filter(level=2, gid_0=gid_0, gid_1=gid_1)
        data = AdminBoundarySerializer(countries, many=True).data
        return Response(data)


class GeostoreViewSet(viewsets.ViewSet):
    renderer_classes = [JSONRenderer]

    @action(detail=True, methods=['post'])
    def post(self, request):

        payload = request.data
        geojson = payload.get("geojson")

        # extract the MultiPolygon geometry from the GeoJSON
        geometry = geojson['geometry']
        geom = GEOSGeometry(json.dumps(geometry))

        if geom.geom_type == "Polygon":
            geom = MultiPolygon(geom)

        # create a new Geostore object and save it to the database
        geostore = Geostore(geom=geom)
        geostore.save()

        res_data = GeostoreSerializer(geostore).data

        return Response(res_data)

    @action(detail=True, methods=['get'])
    @method_decorator(revalidate_cache)
    @method_decorator(cache_page)
    def get(self, request, geostore_id):
        try:
            geostore = Geostore.objects.get(id=geostore_id)
            res_data = GeostoreSerializer(geostore).data
            return Response(res_data)
        except Geostore.DoesNotExist:
            raise NotFound(detail='Geostore not found')

    @action(detail=True, methods=['get'])
    @method_decorator(revalidate_cache)
    @method_decorator(cache_page)
    def get_by_admin(self, request, gid_0, gid_1=None, gid_2=None):
        abm_settings = AdminBoundarySettings.for_request(request)
        data_source = abm_settings.data_source

        simplify_thresh = request.GET.get("thresh")

        geostore_filter = {
            "iso": gid_0,
            "id1": None,
            "id2": None,
        }

        boundary_filter = {
            "gid_0": gid_0,
            "level": 0
        }

        if data_source != "gadm41":
            if gid_1:
                geostore_filter.update({"id1": gid_1})
                boundary_filter.update({"gid_1": gid_1, "level": 1})
            if gid_2:
                geostore_filter.update({"id2": gid_2})
                boundary_filter.update({"gid_2": gid_2, "level": 2})
        else:
            if gid_1:
                geostore_filter.update({"id1": gid_1})
                boundary_filter.update({"gid_1": f"{gid_0}.{gid_1}_1", "level": 1})
            if gid_2:
                geostore_filter.update({"id2": gid_2})
                boundary_filter.update({"gid_2": f"{gid_0}.{gid_1}.{gid_2}_1", "level": 2})

        geostore = Geostore.objects.filter(**geostore_filter)
        should_save = False

        if not geostore.exists():
            should_save = True
            geostore = AdminBoundary.objects.filter(**boundary_filter)

        if not geostore.exists():
            raise NotFound(detail='Geostore not found')

        geostore = geostore.first()

        geom = geostore.geom

        if simplify_thresh:
            geom = geostore.geom.simplify(tolerance=float(simplify_thresh))

        # convert to multipolygon if not
        if geom.geom_type != "MultiPolygon":
            geom = MultiPolygon(geom)

        if should_save:
            geostore_data = {
                "iso": geostore.gid_0,
                "id1": gid_1,
                "id2": gid_2,
                "name_0": geostore.name_0,
                "name_1": geostore.name_1,
                "name_2": geostore.name_2,
                "geom": geom
            }

            geostore = Geostore.objects.create(**geostore_data)

        res_data = GeostoreSerializer(geostore).data

        return Response(res_data)



# Generated by Django 4.1.10 on 2024-01-16 09:53

from django.db import migrations, models
import geomanager.validators


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0040_alter_rasterfilelayer_options_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataset',
            name='order',
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='rasterfilelayer',
            name='auto_ingest_custom_directory_name',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True, validators=[geomanager.validators.validate_directory_name], verbose_name='Custom directory name'),
        ),
    ]



# Generated by Django 4.2.10 on 2024-02-28 14:25

import django.core.validators
from django.db import migrations
import wagtail.blocks
import wagtail.fields
import wagtail_color_panel.blocks


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0043_additionalmapboundarydata'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='additionalmapboundarydata',
            options={'verbose_name': 'Additional Map Boundary Dataset', 'verbose_name_plural': 'Additional Map Boundary Datasets'},
        ),
        migrations.AddField(
            model_name='additionalmapboundarydata',
            name='render_layers',
            field=wagtail.fields.StreamField([('fill', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('fill_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('fill_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('fill_outline_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('fill_antialias', wagtail.blocks.BooleanBlock(default=True, required=False))], label='Mapbox GL Paint Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False))], label='Polygon Layer')), ('line', wagtail.blocks.StructBlock([('paint', wagtail.blocks.StructBlock([('line_color', wagtail_color_panel.blocks.NativeColorBlock(default='#000000', required=False)), ('line_dasharray', wagtail.blocks.CharBlock(required=False)), ('line_gap_width', wagtail.blocks.FloatBlock(default=0.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_opacity', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(1)])), ('line_width', wagtail.blocks.FloatBlock(default=1.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_offset', wagtail.blocks.FloatBlock(default=0, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Paint Properties')), ('layout', wagtail.blocks.StructBlock([('line_cap', wagtail.blocks.ChoiceBlock(choices=[('butt', 'Butt'), ('round', 'Round'), ('square', 'Square')], required=False)), ('line_join', wagtail.blocks.ChoiceBlock(choices=[('miter', 'Miter'), ('bevel', 'Bevel'), ('round', 'Round')], required=False)), ('line_miter_limit', wagtail.blocks.FloatBlock(default=2.0, required=False, validators=[django.core.validators.MinValueValidator(0)])), ('line_round_limit', wagtail.blocks.FloatBlock(default=1.05, required=False, validators=[django.core.validators.MinValueValidator(0)]))], label='Mapbox GL Layout Properties')), ('filter', wagtail.blocks.CharBlock(required=False)), ('maxzoom', wagtail.blocks.IntegerBlock(required=False)), ('minzoom', wagtail.blocks.IntegerBlock(required=False))], label='Line Layer'))], blank=True, null=True, use_json_field=True, verbose_name='Map Layers'),
        ),
    ]



# Generated by Django 4.1.10 on 2023-11-30 14:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0038_rasterfilelayer_legend_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='rasterfilelayer',
            name='order',
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='rastertilelayer',
            name='order',
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='vectorfilelayer',
            name='order',
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='vectortilelayer',
            name='order',
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='wmslayer',
            name='order',
            field=models.IntegerField(blank=True, editable=False, null=True),
        ),
    ]



# Generated by Django 4.1.10 on 2023-07-25 09:50

from django.db import migrations, models
import django.db.models.deletion
import modelcluster.fields


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0010_remove_stationsettings_popup_props'),
    ]

    operations = [
        migrations.CreateModel(
            name='VectorLayerIcon',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255)),
                ('color', models.CharField(max_length=100, null=True)),
                ('file', models.FileField(upload_to='vector_icons/')),
                ('layer', modelcluster.fields.ParentalKey(on_delete=django.db.models.deletion.CASCADE, related_name='icons', to='geomanager.vectorlayer')),
            ],
        ),
    ]



from functools import reduce
from operator import or_

from django import VERSION as DJANGO_VERSION
from django.contrib.admin.options import IncorrectLookupParameters
from django.core.exceptions import (
    ImproperlyConfigured,
    SuspiciousOperation,
)
from django.db import models
from wagtail_modeladmin.views import IndexView


def build_q_object_from_lookup_parameters(parameters):
    q_object = models.Q()
    # The original Django implementation of this method uses parameters.items()
    # which returns param_item_list as a non-list if the parameter is a single item.
    # We need to consistently have a list
    for param, param_item_list in parameters.lists():
        q_object &= reduce(or_, (models.Q((param, item)) for item in param_item_list))
    return q_object


class PatchedIndexView(IndexView):
    def get_queryset(self, request=None):
        request = request or self.request
        
        # First, we collect all the declared list filters.
        (
            self.filter_specs,
            self.has_filters,
            remaining_lookup_params,
            filters_use_distinct,
        ) = self.get_filters(request)
        
        # Then, we let every list filter modify the queryset to its liking.
        qs = self.get_base_queryset(request)
        for filter_spec in self.filter_specs:
            new_qs = filter_spec.queryset(request, qs)
            if new_qs is not None:
                qs = new_qs
        
        try:
            # Finally, we apply the remaining lookup parameters from the query
            # string (i.e. those that haven't already been processed by the
            # filters).
            if DJANGO_VERSION >= (5, 0):
                qs = qs.filter(
                    build_q_object_from_lookup_parameters(remaining_lookup_params)
                )
            else:
                qs = qs.filter(**remaining_lookup_params)
        except (SuspiciousOperation, ImproperlyConfigured):
            # Allow certain types of errors to be re-raised as-is so that the
            # caller can treat them in a special way.
            raise
        except Exception as e:  # noqa: BLE001
            # Every other error is caught with a naked except, because we don't
            # have any other way of validating lookup parameters. They might be
            # invalid if the keyword arguments are incorrect, or if the values
            # are not in the correct type, so we might get FieldError,
            # ValueError, ValidationError, or ?.
            raise IncorrectLookupParameters(e)
        
        if not qs.query.select_related:
            qs = self.apply_select_related(qs)
        
        # Set ordering.
        ordering = self.get_ordering(request, qs)
        qs = qs.order_by(*ordering)
        
        # Remove duplicates from results, if necessary
        if filters_use_distinct:
            qs = qs.distinct()
        
        # Apply search results
        return self.get_search_results(request, qs, self.query)



from allauth.account.forms import ResetPasswordForm
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.utils.module_loading import import_string
from rest_framework import generics
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework_simplejwt.exceptions import TokenError, InvalidToken
from rest_framework_simplejwt.settings import api_settings
from rest_framework_simplejwt.tokens import AccessToken
from rest_framework_simplejwt.views import TokenObtainPairView, TokenVerifyView

from geomanager.serializers import RegisterSerializer, ResetPasswordSerializer
from geomanager.serializers.auth import EmailTokenObtainSerializer


class RegisterView(generics.CreateAPIView):
    queryset = get_user_model().objects.all()
    permission_classes = (AllowAny,)
    serializer_class = RegisterSerializer

    def perform_create(self, serializer):
        email = serializer.validated_data['username']

        # check if user exists
        if get_user_model().objects.filter(email=email).exists():
            raise ValidationError({"email": "User with this email already exists"})

        user = get_user_model().objects.create(
            username=email,
            email=email
        )
        user.save()
        form = ResetPasswordForm({"email": user.email})
        if form.is_valid():
            form.save(self.request)

        return user


class ResetPasswordView(generics.CreateAPIView):
    queryset = get_user_model().objects.all()
    permission_classes = (AllowAny,)
    serializer_class = ResetPasswordSerializer

    def perform_create(self, serializer):
        email = serializer.validated_data['username']

        try:
            user = get_user_model().objects.get(email=email)
            form = ResetPasswordForm({"email": user.email})
            if form.is_valid():
                form.save(self.request)

            return user
        except ObjectDoesNotExist:
            raise ValidationError({"email": "User with this email does not exist"})


class EmailTokenObtainPairView(TokenObtainPairView):
    permission_classes = [AllowAny]
    serializer_class = EmailTokenObtainSerializer


class UserTokenVerifyView(TokenVerifyView):
    def post(self, request, *args, **kwargs):

        serializer = self.get_serializer(data=request.data)

        try:
            serializer.is_valid(raise_exception=True)
        except TokenError as e:
            if request.data.get("refresh"):
                try:
                    serializer_cls = import_string(api_settings.TOKEN_REFRESH_SERIALIZER)
                    serializer = serializer_cls(data=request.data)
                    serializer.is_valid(raise_exception=True)
                except TokenError as e:
                    raise InvalidToken(e.args[0])
            else:
                raise InvalidToken(e.args[0])

        if serializer.validated_data:
            token = AccessToken(serializer.validated_data.get("access"))
        else:
            token = AccessToken(request.data.get("token"))

        user_id = token.get("user_id")
        try:
            user = get_user_model().objects.get(id=user_id)
            user_details = {"email": user.email, "id": user.id}
        except ObjectDoesNotExist:
            return Response({"detail": "user does not exist"}, status=404)

        return Response(user_details)



# Generated by Django 4.1.10 on 2023-07-11 08:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0008_stationsettings_name_column'),
    ]

    operations = [
        migrations.AddField(
            model_name='stationsettings',
            name='popup_props',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



MAPBOX_GL_STYLE_SPEC = {
    "PAINT_DEFAULTS": {
        "fill_color": "#000000",
        "fill_opacity": 1,
        "fill_outline_color": "#000000",
        "fill_antialias": True,

        "line_color": "#000000",
        "line_dasharray": "",
        "line_gap_width": 0,
        "line_opacity": 1,
        "line_width": 1,
        "line_offset": 0,

        "icon_color": "#000000",
        "icon_halo_blur": 0,
        "icon_halo_color": "#000000",
        "icon_halo_width": 0,
        "icon_opacity": 1,

        "text_color": "#000000",
        "text_halo_color": 0,
        "text_halo_width": 0,
        "text_translate": "",
        "text_translate_anchor": "map",

        "circle_color": "#000000",
        "circle_opacity": 1,
        "circle_radius": 5,
        "circle_stroke_color": "#000000",
        "circle_stroke_width": 0,
    },
    "LAYOUT_DEFAULTS": {
        "line_cap": "butt",
        "line_join": "miter",
        "line_miter_limit": 2,
        "line_round_limit": 1.05,

        "icon_allow_overlap": False,
        "icon_anchor": "center",
        "icon_ignore_placement": False,
        "icon_keep_upright": False,
        "icon_offset": "",
        "icon_optional": False,
        "icon_padding": 2,
        "icon_pitch_alignment": "auto",
        "icon_rotate": 0,
        "icon_rotation_alignment": "auto",
        "icon_size": 1,
        "icon_text_fit": "none",

        "symbol_placement": "point",
        "text_allow_overlap": False,
        "text_anchor": "center",
        "text_size": 16,
        "text_transform": "none",
        "text_ignore_placement": False,
        "text_justify": "center",
        "text_keep_upright": False,
        "text_letter_spacing": 0,
        "text_line_height": 1.2,
        "text_max_angle": 45,
        "text_max_width": 10,
        "text_offset": "",
        "text_opacity": 1,
        "text_padding": 2,
        "text_pitch_alignment": "auto",
        "text_radial_offset": 0,
        "text_rotate": 0,
        "text_rotation_alignment": "auto",
        "text_variable_anchor": "",
    }
}

DEFAULT_OPEN_MAP_TILES_STYLE = {
    "version": 8,
    "name": "Base",
    "metadata": {
        "openmaptiles:version": "3.x",
        "mapbox:groups": {
            "basemap_light": {
                "name": "basemap-light"
            },
            "basemap_dark": {
                "name": "basemap-dark"
            },
            "basemap_satellite": {
                "name": "basemap-satellite"
            },
            "labels_light": {
                "name": "labels-light"
            },
            "labels_dark": {
                "name": "labels-dark"
            },
            "roads": {
                "name": "roads"
            }
        }
    },
    "sources": {
        "openmaptiles": {
            "type": "vector",
            "url": ""
        }
    },
    "layers": [
        {
            "id": "background-light",
            "paint": {
                "background-color": "hsl(0, 0%, 100%)"
            },
            "layout": {
                "visibility": "none"
            },
            "type": "background",
            "metadata": {
                "mapbox:group": "basemap_light"
            }
        },
        {
            "id": "background-dark",
            "type": "background",
            "metadata": {
                "mapbox:group": "basemap_dark"
            },
            "layout": {
                "visibility": "none"
            },
            "paint": {
                "background-color": "hsl(60, 2%, 19%)"
            }
        },
        {
            "id": "background-satellite",
            "type": "background",
            "metadata": {
                "mapbox:group": "basemap_satellite"
            },
            "layout": {
                "visibility": "none"
            },
            "paint": {
                "background-color": "#fff"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Polygon"
                ],
                [
                    "!=",
                    "intermittent",
                    1
                ]
            ],
            "id": "water",
            "paint": {
                "fill-color": "hsl(203, 95%, 84%)"
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "water",
            "type": "fill",
            "metadata": {
                "mapbox:group": "basemap_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Polygon"
                ],
                [
                    "!=",
                    "intermittent",
                    1
                ]
            ],
            "id": "water_dark",
            "paint": {
                "fill-color": "hsl(60, 5%, 12%)"
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "water",
            "type": "fill",
            "metadata": {
                "mapbox:group": "basemap_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Polygon"
                ],
                [
                    "==",
                    "intermittent",
                    1
                ]
            ],
            "id": "water_intermittent",
            "paint": {
                "fill-color": "hsl(203, 95%, 84%)",
                "fill-opacity": 0.7
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "water",
            "type": "fill",
            "metadata": {
                "mapbox:group": "basemap_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Polygon"
                ],
                [
                    "==",
                    "intermittent",
                    1
                ]
            ],
            "id": "water_intermittent_dark",
            "paint": {
                "fill-color": "hsl(60, 5%, 12%)",
                "fill-opacity": 0.7
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "water",
            "type": "fill",
            "metadata": {
                "mapbox:group": "basemap_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "!in",
                    "brunnel",
                    "tunnel",
                    "bridge"
                ],
                [
                    "!=",
                    "intermittent",
                    1
                ]
            ],
            "id": "waterway",
            "paint": {
                "line-color": "hsl(203, 95%, 84%)",
                "line-opacity": 1,
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            8,
                            1
                        ],
                        [
                            20,
                            8
                        ]
                    ]
                }
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "waterway",
            "minzoom": 8,
            "type": "line",
            "metadata": {
                "mapbox:group": "basemap_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "!in",
                    "brunnel",
                    "tunnel",
                    "bridge"
                ],
                [
                    "!=",
                    "intermittent",
                    1
                ]
            ],
            "id": "waterway_dark",
            "paint": {
                "line-color": "hsl(0, 0%, 0%)",
                "line-opacity": 1,
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            8,
                            1
                        ],
                        [
                            20,
                            8
                        ]
                    ]
                }
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "waterway",
            "minzoom": 8,
            "type": "line",
            "metadata": {
                "mapbox:group": "basemap_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "!in",
                    "brunnel",
                    "tunnel",
                    "bridge"
                ],
                [
                    "==",
                    "intermittent",
                    1
                ]
            ],
            "id": "waterway_intermittent",
            "paint": {
                "line-color": "hsl(203, 95%, 84%)",
                "line-opacity": 1,
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            8,
                            1
                        ],
                        [
                            20,
                            8
                        ]
                    ]
                },
                "line-dasharray": [
                    2,
                    1
                ]
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "waterway",
            "minzoom": 8,
            "type": "line",
            "metadata": {
                "mapbox:group": "basemap_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "!in",
                    "brunnel",
                    "tunnel",
                    "bridge"
                ],
                [
                    "==",
                    "intermittent",
                    1
                ]
            ],
            "id": "waterway_intermittent_dark",
            "paint": {
                "line-color": "hsl(0, 0%, 0%)",
                "line-opacity": 1,
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            8,
                            1
                        ],
                        [
                            20,
                            8
                        ]
                    ]
                },
                "line-dasharray": [
                    2,
                    1
                ]
            },
            "layout": {
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "waterway",
            "minzoom": 8,
            "type": "line",
            "metadata": {
                "mapbox:group": "basemap_dark"
            }
        },
        {
            "id": "road_pier",
            "type": "line",
            "metadata": {
                "mapbox:group": "roads"
            },
            "source": "openmaptiles",
            "source-layer": "transportation",
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "in",
                    "class",
                    "pier"
                ]
            ],
            "layout": {
                "line-cap": "round",
                "line-join": "round",
                "visibility": "none"
            },
            "paint": {
                "line-color": "#fffca2",
                "line-width": {
                    "base": 1.2,
                    "stops": [
                        [
                            15,
                            1
                        ],
                        [
                            17,
                            4
                        ]
                    ]
                }
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "in",
                    "class",
                    "path",
                    "track"
                ]
            ],
            "id": "road_path",
            "paint": {
                "line-color": "#fffca2",
                "line-dasharray": [
                    1,
                    1
                ],
                "line-width": {
                    "base": 1.55,
                    "stops": [
                        [
                            4,
                            0.25
                        ],
                        [
                            20,
                            10
                        ]
                    ]
                }
            },
            "layout": {
                "line-cap": "square",
                "line-join": "bevel",
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "transportation",
            "type": "line",
            "metadata": {
                "mapbox:group": "roads"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "in",
                    "class",
                    "minor",
                    "service"
                ]
            ],
            "id": "road_minor",
            "paint": {
                "line-color": "#fffca2",
                "line-width": {
                    "base": 1.55,
                    "stops": [
                        [
                            4,
                            0.25
                        ],
                        [
                            20,
                            30
                        ]
                    ]
                }
            },
            "layout": {
                "line-cap": "round",
                "line-join": "round",
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "transportation",
            "type": "line",
            "minzoom": 13,
            "metadata": {
                "mapbox:group": "roads"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "in",
                    "class",
                    "trunk",
                    "primary"
                ]
            ],
            "id": "road_trunk_primary",
            "paint": {
                "line-color": "#fffca2",
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            6,
                            0.5
                        ],
                        [
                            20,
                            30
                        ]
                    ]
                }
            },
            "layout": {
                "line-cap": "round",
                "line-join": "round",
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "transportation",
            "type": "line",
            "metadata": {
                "mapbox:group": "roads"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "in",
                    "class",
                    "secondary",
                    "tertiary"
                ]
            ],
            "id": "road_secondary_tertiary",
            "paint": {
                "line-color": "#fffca2",
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            6,
                            0.5
                        ],
                        [
                            20,
                            20
                        ]
                    ]
                }
            },
            "layout": {
                "line-cap": "round",
                "line-join": "round",
                "visibility": "none"
            },
            "source": "openmaptiles",
            "source-layer": "transportation",
            "type": "line",
            "metadata": {
                "mapbox:group": "roads"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "LineString"
                ],
                [
                    "==",
                    "class",
                    "motorway"
                ]
            ],
            "id": "road_major_motorway",
            "layout": {
                "line-cap": "round",
                "line-join": "round",
                "visibility": "none"
            },
            "paint": {
                "line-color": "#fffca2",
                "line-offset": 0,
                "line-width": {
                    "base": 1.4,
                    "stops": [
                        [
                            8,
                            1
                        ],
                        [
                            16,
                            10
                        ]
                    ]
                }
            },
            "source": "openmaptiles",
            "source-layer": "transportation",
            "type": "line",
            "metadata": {
                "mapbox:group": "roads"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "rank",
                    1
                ]
            ],
            "id": "poi_label",
            "layout": {
                "icon-size": 1,
                "text-anchor": "top",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 8,
                "text-offset": [
                    0,
                    0.5
                ],
                "text-size": 11,
                "visibility": "none"
            },
            "minzoom": 14,
            "paint": {
                "text-color": "#666",
                "text-halo-blur": 1,
                "text-halo-color": "rgba(255,255,255,0.75)",
                "text-halo-width": 1,
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "poi",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "rank",
                    1
                ]
            ],
            "id": "poi_label_dark",
            "layout": {
                "icon-size": 1,
                "text-anchor": "top",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 8,
                "text-offset": [
                    0,
                    0.5
                ],
                "text-size": 11,
                "visibility": "none"
            },
            "minzoom": 14,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "poi",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "has",
                    "iata"
                ]
            ],
            "id": "airport-label",
            "layout": {
                "icon-size": 1,
                "text-anchor": "top",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 8,
                "text-offset": [
                    0,
                    0.5
                ],
                "text-size": 11,
                "visibility": "none"
            },
            "minzoom": 10,
            "paint": {
                "text-color": "#666",
                "text-halo-blur": 1,
                "text-halo-color": "rgba(255,255,255,0.75)",
                "text-halo-width": 1,
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "aerodrome_label",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "has",
                    "iata"
                ]
            ],
            "id": "airport-label-dark",
            "layout": {
                "icon-size": 1,
                "text-anchor": "top",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 8,
                "text-offset": [
                    0,
                    0.5
                ],
                "text-size": 11,
                "visibility": "none"
            },
            "minzoom": 10,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "aerodrome_label",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        },
        {
            "filter": [
                "==",
                "$type",
                "LineString"
            ],
            "id": "road_major_label",
            "layout": {
                "symbol-placement": "line",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-letter-spacing": 0.1,
                "text-rotation-alignment": "map",
                "text-size": {
                    "base": 1.4,
                    "stops": [
                        [
                            10,
                            8
                        ],
                        [
                            20,
                            14
                        ]
                    ]
                },
                "text-transform": "uppercase",
                "visibility": "none"
            },
            "paint": {
                "text-color": "#000",
                "text-halo-color": "hsl(0, 0%, 100%)",
                "text-halo-width": 2,
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "transportation_name",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "==",
                "$type",
                "LineString"
            ],
            "id": "road_major_label_dark",
            "layout": {
                "symbol-placement": "line",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-letter-spacing": 0.1,
                "text-rotation-alignment": "map",
                "text-size": {
                    "base": 1.4,
                    "stops": [
                        [
                            10,
                            8
                        ],
                        [
                            20,
                            14
                        ]
                    ]
                },
                "text-transform": "uppercase",
                "visibility": "none"
            },
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "transportation_name",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "!in",
                    "class",
                    "city",
                    "state",
                    "country",
                    "continent"
                ]
            ],
            "id": "place_label_other",
            "layout": {
                "text-anchor": "center",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 6,
                "text-size": {
                    "stops": [
                        [
                            6,
                            10
                        ],
                        [
                            12,
                            14
                        ]
                    ]
                },
                "visibility": "none"
            },
            "minzoom": 8,
            "paint": {
                "text-color": "hsl(0, 0%, 25%)",
                "text-halo-blur": 0,
                "text-halo-color": "hsl(0, 0%, 100%)",
                "text-halo-width": 2,
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "!in",
                    "class",
                    "city",
                    "state",
                    "country",
                    "continent"
                ]
            ],
            "id": "place_label_other_dark",
            "layout": {
                "text-anchor": "center",
                "text-field": "{name}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 6,
                "text-size": {
                    "stops": [
                        [
                            6,
                            10
                        ],
                        [
                            12,
                            14
                        ]
                    ]
                },
                "visibility": "none"
            },
            "minzoom": 8,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "class",
                    "city"
                ]
            ],
            "id": "place_label_city",
            "layout": {
                "text-field": "{name_en}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 10,
                "text-size": {
                    "stops": [
                        [
                            3,
                            12
                        ],
                        [
                            8,
                            16
                        ]
                    ]
                },
                "visibility": "none"
            },
            "maxzoom": 16,
            "paint": {
                "text-color": "hsl(0, 0%, 0%)",
                "text-halo-blur": 0,
                "text-halo-color": "hsla(0, 0%, 100%, 0.75)",
                "text-halo-width": 2,
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "class",
                    "city"
                ]
            ],
            "id": "place_label_city_dark",
            "layout": {
                "text-field": "{name_en}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 10,
                "text-size": {
                    "stops": [
                        [
                            3,
                            12
                        ],
                        [
                            8,
                            16
                        ]
                    ]
                },
                "visibility": "none"
            },
            "maxzoom": 16,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "class",
                    "country"
                ],
                [
                    "!has",
                    "iso_a2"
                ]
            ],
            "id": "country_label-other",
            "layout": {
                "text-field": "{name_en}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 10,
                "text-size": {
                    "stops": [
                        [
                            3,
                            12
                        ],
                        [
                            8,
                            22
                        ]
                    ]
                },
                "visibility": "none"
            },
            "maxzoom": 12,
            "paint": {
                "text-color": "hsl(0, 0%, 13%)",
                "text-halo-blur": 0,
                "text-halo-color": "rgba(255,255,255,0.75)",
                "text-halo-width": 2,
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "class",
                    "country"
                ],
                [
                    "!has",
                    "iso_a2"
                ]
            ],
            "id": "country_label-other_dark",
            "layout": {
                "text-field": "{name_en}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 10,
                "text-size": {
                    "stops": [
                        [
                            3,
                            12
                        ],
                        [
                            8,
                            22
                        ]
                    ]
                },
                "visibility": "none"
            },
            "maxzoom": 12,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "class",
                    "country"
                ],
                [
                    "has",
                    "iso_a2"
                ]
            ],
            "id": "country_label",
            "layout": {
                "text-field": "{name_en}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 10,
                "text-size": {
                    "base": 1,
                    "stops": [
                        [
                            1,
                            10
                        ],
                        [
                            6,
                            24
                        ]
                    ]
                },
                "visibility": "none"
            },
            "maxzoom": 12,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": {
                    "base": 1,
                    "stops": [
                        [
                            2,
                            "rgba(255,255,255,0.75)"
                        ],
                        [
                            3,
                            "#fff"
                        ]
                    ]
                },
                "text-color": "hsl(0, 0%, 0%)",
                "text-opacity": 0.7
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_light"
            }
        },
        {
            "filter": [
                "all",
                [
                    "==",
                    "$type",
                    "Point"
                ],
                [
                    "==",
                    "class",
                    "country"
                ],
                [
                    "has",
                    "iso_a2"
                ]
            ],
            "id": "country_label_dark",
            "layout": {
                "text-field": "{name_en}",
                "text-font": [
                    "Klokantech Noto Sans Regular"
                ],
                "text-max-width": 10,
                "text-size": {
                    "base": 1,
                    "stops": [
                        [
                            1,
                            10
                        ],
                        [
                            6,
                            24
                        ]
                    ]
                },
                "visibility": "none"
            },
            "maxzoom": 12,
            "paint": {
                "text-halo-width": 1.25,
                "text-halo-color": "hsla(0, 0%, 10%, 0.75)",
                "text-color": "hsl(0, 0%, 100%)",
                "text-opacity": 0.9
            },
            "source": "openmaptiles",
            "source-layer": "place",
            "type": "symbol",
            "metadata": {
                "mapbox:group": "labels_dark"
            }
        }
    ],
    "id": "base"
}



from django import template

register = template.Library()


@register.simple_tag()
def get_object_attr(obj, attr):
    if hasattr(obj, attr):
        return getattr(obj, attr)
    return None


@register.inclusion_tag("geomanager/shared/breadcrumbs.html")
def breadcrumbs_component(items, is_expanded=True, classname=None):
    return {"items": items, "is_expanded": is_expanded, "classname": classname}



# Generated by Django 4.2.10 on 2024-06-18 09:22

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0050_geomanagersettings_contact_us_page'),
    ]

    operations = [
        migrations.AddField(
            model_name='rasterstyle',
            name='rendering_engine',
            field=models.CharField(choices=[('large_image', 'Default'), ('magics', 'ECMWF Magics')], default='large_image', max_length=100, verbose_name='Rendering Engine'),
        ),
    ]



from django.conf import settings

DEFAULT_NC_TIME_DIMENSION_NAMES = [
    "time",
    "TIME",
    "XTIME",
    "xtime",
]

EXTRA_NC_TIME_DIMENSION_NAMES = getattr(settings, "GEOMANAGER_EXTRA_NC_TIME_DIMENSION_NAMES", [])

NC_TIME_DIMENSION_NAMES = DEFAULT_NC_TIME_DIMENSION_NAMES + EXTRA_NC_TIME_DIMENSION_NAMES

geomanager_settings = {
    "vector_db_schema": getattr(settings, "GEOMANAGER_VECTOR_DB_SCHEMA", "vectordata"),
    "auto_ingest_raster_data_dir": getattr(settings, "GEOMANAGER_AUTO_INGEST_RASTER_DATA_DIR", None),
    "nc_time_dimension_names": NC_TIME_DIMENSION_NAMES,
}



# Generated by Django 4.1.10 on 2024-01-09 10:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0039_rasterfilelayer_order_rastertilelayer_order_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='rasterfilelayer',
            options={'ordering': ['order'], 'verbose_name': 'Raster File Layer', 'verbose_name_plural': 'Raster File Layers'},
        ),
        migrations.AlterModelOptions(
            name='rastertilelayer',
            options={'ordering': ['order'], 'verbose_name': 'Raster Tile Layer', 'verbose_name_plural': 'Raster Tile Layers'},
        ),
        migrations.AlterModelOptions(
            name='vectorfilelayer',
            options={'ordering': ['order'], 'verbose_name': 'Vector File Layer', 'verbose_name_plural': 'Vector File Layers'},
        ),
        migrations.AlterModelOptions(
            name='vectortilelayer',
            options={'ordering': ['order'], 'verbose_name': 'Vector Tile Layer', 'verbose_name_plural': 'Vector Tile Layers'},
        ),
        migrations.AlterModelOptions(
            name='wmslayer',
            options={'ordering': ['order'], 'verbose_name': 'WMS Layer', 'verbose_name_plural': 'WMS Layers'},
        ),
        migrations.AddField(
            model_name='rasterfilelayer',
            name='auto_ingest_custom_directory_name',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True, verbose_name='Custom directory name'),
        ),
        migrations.AddField(
            model_name='rasterfilelayer',
            name='auto_ingest_use_custom_directory_name',
            field=models.BooleanField(default=False, verbose_name='Use custom directory name'),
        ),
    ]



from django.db import models
from django.db.models.signals import pre_delete
from django.dispatch import receiver
from django.urls import reverse
from django.utils.translation import gettext_lazy as _
from wagtail.api.v2.utils import get_full_url
from wagtail.fields import StreamField

from geomanager.blocks import FillVectorLayerBlock, LineVectorLayerBlock
from geomanager.fields import ListField
from geomanager.settings import geomanager_settings
from geomanager.utils.tiles import get_vector_render_layers
from geomanager.utils.vector_utils import drop_vector_table


class AdditionalMapBoundaryData(models.Model):
    name = models.CharField(max_length=255, unique=True, verbose_name=_("Dataset Name"))
    table_name = models.CharField(max_length=256, unique=True)

    properties = models.JSONField()
    geometry_type = models.CharField(max_length=100)
    bounds = ListField(max_length=256)
    active = models.BooleanField(default=True, verbose_name=_("Active"))

    render_layers = StreamField([
        ("fill", FillVectorLayerBlock(label=_("Polygon Layer"))),
        ("line", LineVectorLayerBlock(label=_("Line Layer"))),
    ], use_json_field=True, null=True, blank=True, min_num=1, verbose_name=_("Map Layers"))

    class Meta:
        verbose_name = _("Additional Map Boundary Dataset")
        verbose_name_plural = _("Additional Map Boundary Datasets")

    def __str__(self):
        return self.name

    @property
    def tiles_url(self):
        tiles_url = reverse("map_boundary_tiles", args=(self.table_name, 0, 0, 0)).replace("/0/0/0", r"/{z}/{x}/{y}")
        return tiles_url

    @property
    def columns(self):
        if not self.properties:
            return []
        if self.properties:
            return [c.get("name") for c in self.properties]

    @property
    def full_table_name(self):
        pg_service_schema = geomanager_settings.get("vector_db_schema")
        return f"{pg_service_schema}.{self.table_name}"

    def get_dataset_config(self, request=None):
        dataset_id = f"{self.table_name}-{self.pk}"
        tiles_url = get_full_url(request, self.tiles_url)

        render_layers = get_vector_render_layers(self.render_layers)

        dataset_config = {
            "id": dataset_id,
            "dataset": dataset_id,
            "name": self.name,
            "layer": dataset_id,
            "isBoundary": True,
            "public": True,
            "layers": [
                {
                    "id": dataset_id,
                    "isBoundary": True,
                    "analysisEndpoint": "use",
                    "tableName": self.table_name,
                    "name": self.name,
                    "default": True,
                    "layerConfig": {
                        "type": "vector",
                        "source": {
                            "type": "vector",
                            "tiles": [tiles_url],
                        },
                        "render": {
                            "layers": render_layers
                        }
                    },
                    "interactionConfig": {
                        "output": [
                            {
                                "column": "gid",
                                "property": "ID",
                                'type': "string",
                            },
                        ]
                    }
                }
            ]
        }

        return dataset_config


@receiver(pre_delete, sender=AdditionalMapBoundaryData)
def drop_pg_vector_table(sender, instance, **kwargs):
    if instance.table_name:
        drop_vector_table(instance.table_name)



# Generated by Django 4.2.10 on 2024-04-23 08:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0048_alter_geomanagersettings_map_disclaimer_text'),
    ]

    operations = [
        migrations.AddField(
            model_name='geomanagersettings',
            name='allow_signups',
            field=models.BooleanField(default=False, verbose_name='Allow user signups'),
        ),
        migrations.AddField(
            model_name='geomanagersettings',
            name='enable_my_account',
            field=models.BooleanField(default=False, verbose_name='Enable My Account'),
        ),
    ]



from asgiref.sync import async_to_sync
from django_nextjs.render import render_nextjs_page
from wagtail.api.v2.utils import get_full_url
from wagtailiconchooser.utils import get_svg_sprite_for_icons

from geomanager.models import GeomanagerSettings, Category
from geomanager.models.vector_file import get_legend_icons


def map_view(request, location_type=None, adm0=None, adm1=None, adm2=None):
    # get svg sprite for categories and legend icons
    category_icons = [category.icon for category in Category.objects.all()]
    legend_icons = get_legend_icons()
    icons = [*category_icons, *legend_icons]
    svg_sprite = get_svg_sprite_for_icons(icons)

    context = {
        "svg_sprite": svg_sprite,
    }

    return async_to_sync(render_nextjs_page)(request, "django_nextjs/mapviewer.html", context, allow_redirects=True)



import json
import math
from uuid import UUID

from django.utils.translation import gettext_lazy as _


class UUIDEncoder(json.JSONEncoder):
    def default(self, obj):
        if isinstance(obj, UUID):
            # if the obj is uuid, we simply return the value of uuid
            return str(obj)
        return json.JSONEncoder.default(self, obj)


DATE_FORMAT_CHOICES = (
    ("yyyy-MM-dd HH:mm", _("Hour minute:second - (E.g 2023-01-01 00:00)")),
    ("yyyy-MM-dd", _("Day - (E.g 2023-01-01)")),
    ("pentadal", _("Pentadal - (E.g Jan 2023 - P1 1-5th)")),
    ("dekadal", _("Dekadal - (E.g Jan 2023 - D1 1-10th)")),
    ("yyyy-MM", _("Month number - (E.g 2023-01)")),
    ("MMMM yyyy", _("Month name - (E.g January 2023)")),
    ("yyyy", _("Year - (E.g 2023)")),
)


def significant_digits(step, number):
    """
    This function calculates the number of significant digits to keep after the decimal
    point based on a given step value.

    Args:
        step: The minimum absolute difference between two values considered significant.
        number: The number to analyze.


    Returns:
        The number of significant digits to keep.
    """
    if number == 0:
        return 1  # Handle zero case

    # Convert the number to absolute value and handle negatives
    abs_number = abs(number)
    # Get the integer part (number of leading zeros not considered significant)
    exponent = int(math.floor(math.log10(abs_number)))

    # If the number is less than 1 (between 0 and 1), consider the leading zero
    if abs_number < 1:
        exponent -= 1

    # Calculate significant digits based on exponent and step
    sign_digits = max(0, exponent + 1)  # At least 1 significant digit
    if step > 0:
        sign_digits = max(sign_digits, int(math.ceil(math.log10(abs_number) / math.log10(step))))

    return sign_digits


def round_to_precision(precision=None):
    """
    This function creates and returns another function that rounds numbers to a specified precision.

    Args:
        precision: The desired number of decimal places for rounding (optional).

    Returns:
        A function that rounds numbers to the specified precision.
    """
    if precision is None:
        def inner(n):
            return n  # No rounding if precision is not specified

        return inner

    multiplier = 10 ** precision

    def inner(n):
        """
        Inner function that performs rounding based on the multiplier.
        """
        return round(n * multiplier) / multiplier

    return inner



import glob
import logging
import os

from django.core.management.base import BaseCommand

from geomanager.settings import geomanager_settings
from geomanager.utils.ingest import is_valid_uuid, ingest_raster_file

logger = logging.getLogger(__name__)

ALLOWED_FILE_EVENTS = ["created", "moved"]

RASTER_FILE_EXTENSIONS = ['.tif', '.nc', ]


class Command(BaseCommand):
    help = 'Process a raster file layer directory'

    def add_arguments(self, parser):
        parser.add_argument('layer_id', type=str, help="Layer ID")
        parser.add_argument('--overwrite', action='store_true', default=False, help='Overwrite existing raster file')
        parser.add_argument('--clip', action='store_true', default=False, help='Clip raster to county boundary')

    def handle(self, *args, **options):
        layer_id = options['layer_id']
        overwrite = options['overwrite']
        clip = options['clip']

        auto_ingest_raster_data_dir = geomanager_settings.get("auto_ingest_raster_data_dir", None)

        if not auto_ingest_raster_data_dir:
            logger.error("[GEOMANAGER_AUTO_INGEST]: Auto ingest raster data directory not configured.")
            return

        logger.debug('[GEOMANAGER_AUTO_INGEST]: Starting directory processing...')

        directory = os.path.join(auto_ingest_raster_data_dir, layer_id)

        if not os.path.isdir(directory):
            logger.error(f"[GEOMANAGER_AUTO_INGEST]: Directory: {directory} does not exist.")
            return

        for ext in RASTER_FILE_EXTENSIONS:
            files = glob.glob(directory + f"/*{ext}")

            if not files:
                logger.warning(
                    f"[GEOMANAGER_AUTO_INGEST]: No files found in directory: {directory} with extension: {ext}")

            for file in files:
                logger.info(f'[GEOMANAGER_AUTO_INGEST]: Processing file: {file}')
                ingest_raster_file(file, overwrite, clip)
                logger.info(f'[GEOMANAGER_AUTO_INGEST]: {file} done...')



from rest_framework import serializers

from geomanager.models import RasterTileLayer


class RasterTileLayerSerializer(serializers.ModelSerializer):
    layerConfig = serializers.SerializerMethodField()
    params = serializers.SerializerMethodField()
    paramsSelectorConfig = serializers.SerializerMethodField()
    legendConfig = serializers.SerializerMethodField()
    name = serializers.SerializerMethodField()
    layerType = serializers.SerializerMethodField()
    multiTemporal = serializers.SerializerMethodField()
    currentTimeMethod = serializers.SerializerMethodField()
    paramsSelectorColumnView = serializers.SerializerMethodField()
    autoUpdateInterval = serializers.SerializerMethodField()
    isMultiLayer = serializers.SerializerMethodField()
    nestedLegend = serializers.SerializerMethodField()
    moreInfo = serializers.SerializerMethodField()
    tileJsonUrl = serializers.SerializerMethodField()
    timestampsResponseObjectKey = serializers.SerializerMethodField()
    isDefault = serializers.SerializerMethodField()
    linkedLayers = serializers.SerializerMethodField()
    showAllMultiLayer = serializers.SerializerMethodField()

    class Meta:
        model = RasterTileLayer
        fields = ["id", "dataset", "isDefault", "name", "isMultiLayer", "nestedLegend", "layerType", "layerConfig",
                  "params", "paramsSelectorConfig", "paramsSelectorColumnView", "legendConfig", "multiTemporal",
                  "currentTimeMethod", "autoUpdateInterval", "moreInfo", "tileJsonUrl", "timestampsResponseObjectKey",
                  "linkedLayers", "showAllMultiLayer"]

    def get_showAllMultiLayer(self, obj):
        return obj.dataset.enable_all_multi_layers_on_add

    def get_linkedLayers(self, obj):
        return obj.linked_layers

    def get_isDefault(self, obj):
        return obj.default

    def get_isMultiLayer(self, obj):
        return obj.dataset.multi_layer

    def get_nestedLegend(self, obj):
        return obj.dataset.multi_layer

    def get_autoUpdateInterval(self, obj):
        return obj.dataset.auto_update_interval_milliseconds

    def get_multiTemporal(self, obj):
        return obj.dataset.multi_temporal and obj.has_time

    def get_currentTimeMethod(self, obj):
        return obj.dataset.current_time_method

    def get_layerType(self, obj):
        return obj.dataset.layer_type

    def get_name(self, obj):
        return obj.title

    def get_layerConfig(self, obj):
        layer_config = obj.layer_config

        return layer_config

    def get_params(self, obj):
        return obj.params

    def get_paramsSelectorConfig(self, obj):
        return obj.param_selector_config

    def get_paramsSelectorColumnView(self, obj):
        return not obj.params_selectors_side_by_side

    def get_tileJsonUrl(self, obj):
        if obj.has_time:
            return obj.tile_json_url
        return None

    def get_timestampsResponseObjectKey(self, obj):
        if obj.has_time:
            return obj.timestamps_response_object_key
        return None

    def get_timestampsResponseObjectKey(self, obj):
        if obj.has_time:
            return obj.timestamps_response_object_key
        return None

    def get_legendConfig(self, obj):
        request = self.context.get('request')
        return obj.get_legend_config(request)

    def get_moreInfo(self, obj):
        info = None
        for info in obj.more_info:
            info = info.value.as_dict
        return info



from django.db import models
from django.utils.translation import gettext_lazy as _
from wagtail.admin.panels import FieldPanel

from geomanager.models.core import Dataset
from geomanager.models.tile_base import BaseTileLayer


class RasterTileLayer(BaseTileLayer):
    dataset = models.ForeignKey(Dataset, on_delete=models.CASCADE, related_name="raster_tile_layers",
                                verbose_name=_("dataset"))

    class Meta:
        verbose_name = _("Raster Tile Layer")
        verbose_name_plural = _("Raster Tile Layers")
        ordering = ['order']

    panels = [
        FieldPanel("dataset"),
        *BaseTileLayer.panels,
    ]

    def __str__(self):
        return self.title

    @property
    def layer_config(self):
        tile_url = self.tile_url

        layer_config = {
            "type": "raster",
            "source": {
                "type": "raster",
                "tiles": [tile_url]
            }
        }

        return layer_config



from django import forms
from django.utils.translation import gettext_lazy as _


class LayerRasterFileForm(forms.Form):
    layer = forms.ModelChoiceField(required=True, queryset=None, empty_label=None, label=_("layer"), )
    time = forms.DateTimeField(required=True,
                               widget=forms.DateTimeInput(attrs={'type': 'datetime-local'}, ),
                               localize=False, label=_("time"),
                               help_text=_("Time for the raster file. This can be the time the data was acquired, "
                                           "or the date and time for which the data applies"))
    nc_dates = forms.MultipleChoiceField(required=False, widget=forms.CheckboxSelectMultiple, label=_("NetCDF Dates"),
                                         help_text=_("Timestamps in the dataset"))
    nc_data_variable = forms.CharField(required=False, widget=forms.HiddenInput())

    def __init__(self, nc_dates_choices=None, *args, **kwargs):

        queryset = kwargs.pop('queryset', None)
        super().__init__(*args, **kwargs)
        self.fields['layer'].queryset = queryset

        if nc_dates_choices:
            self.fields['nc_dates'].choices = [(choice, choice) for choice in nc_dates_choices]

            # hide time input
            self.fields['time'].widget = forms.HiddenInput()
            self.fields['time'].required = False
        else:
            self.fields['nc_dates'].widget = forms.HiddenInput()



# Generated by Django 4.1.10 on 2023-11-30 12:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0036_vectortilelayer_popup_config_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataset',
            name='enable_all_multi_layers_on_add',
            field=models.BooleanField(default=True, help_text='Enable all Multi-Layers at once when adding the dataset to the map', verbose_name='Enable all Multi-Layers when adding to map'),
        ),
    ]



# Generated by Django 4.2.10 on 2024-04-23 08:15

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('wagtailcore', '0089_log_entry_data_json_null_to_object'),
        ('geomanager', '0049_geomanagersettings_allow_signups_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='geomanagersettings',
            name='contact_us_page',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='wagtailcore.page', verbose_name='Contact Us Page'),
        ),
    ]



import os
import tempfile

from django import forms
from django.conf import settings
from django.core.exceptions import ValidationError
from django.utils.text import slugify
from django.utils.translation import gettext as _
from wagtail.admin.forms import WagtailAdminModelForm

from geomanager.errors import InvalidGeomType, GeomValidationNotImplemented
from geomanager.models import AdditionalMapBoundaryData
from geomanager.settings import geomanager_settings
from geomanager.utils.vector_utils import ogr_db_import


class AdditionalBoundaryDataAddForm(WagtailAdminModelForm):
    file = forms.FileField(required=True,
                           label=_("Boundary Data"),
                           help_text=_(
                               "Upload a file containing the boundary data. "
                               "The file should be in Shapefile or GeoJSON format."),
                           )

    class Meta:
        model = AdditionalMapBoundaryData
        fields = ["name", "file", "render_layers"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # only accept zipped shapefiles and geojson files
        self.fields["file"].widget.attrs["accept"] = ".zip,.geojson"

    def clean(self):
        cleaned_data = super().clean()
        file = cleaned_data.get("file")
        name = cleaned_data.get("name")
        # slugify and replace dash with underscore
        table_name = slugify(name).replace("-", "_")

        if file:
            _, ext = os.path.splitext(file.name)

            with tempfile.NamedTemporaryFile(suffix=".upload" + ext) as temp_file:
                temp_file.write(file.read())
                temp_file.flush()

                default_db_settings = settings.DATABASES['default']

                db_params = {
                    "host": default_db_settings.get("HOST"),
                    "port": default_db_settings.get("PORT"),
                    "user": default_db_settings.get("USER"),
                    "password": default_db_settings.get("PASSWORD"),
                    "name": default_db_settings.get("NAME"),
                }

                db_settings = {
                    **db_params,
                    "pg_service_schema": geomanager_settings.get("vector_db_schema")
                }

                try:
                    table_info = ogr_db_import(temp_file.name, table_name, db_settings,
                                               validate_geom_types=["Polygon", "MultiPolygon"],
                                               overwrite=True)
                    cleaned_data["table_name"] = table_name
                    cleaned_data["properties"] = table_info.get("properties")
                    cleaned_data["geometry_type"] = table_info.get("geom_type")
                    cleaned_data["bounds"] = table_info.get("bounds")

                except (InvalidGeomType, GeomValidationNotImplemented) as e:
                    raise ValidationError(e)
                except Exception as e:
                    raise ValidationError(e)

        return cleaned_data

    def save(self, commit=True):
        instance = super().save(commit=False)

        # Save the table name and properties
        instance.table_name = self.cleaned_data["table_name"]
        instance.properties = self.cleaned_data["properties"]
        instance.bounds = self.cleaned_data["bounds"]
        instance.geometry_type = self.cleaned_data["geometry_type"]

        if commit:
            instance.save()
        return instance


class AdditionalBoundaryDataEditForm(WagtailAdminModelForm):
    class Meta:
        model = AdditionalMapBoundaryData
        fields = ["name", "active", "render_layers"]



# Generated by Django 4.1.10 on 2023-09-20 08:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('geomanager', '0017_wmslayer_more_info_alter_wmslayer_date_format'),
    ]

    operations = [
        migrations.AddField(
            model_name='wmslayer',
            name='custom_get_capabilities_url',
            field=models.URLField(blank=True, help_text='Alternative URL for the GetCapabilities request', null=True, verbose_name='Get Capabilities URL'),
        ),
    ]
